package LCM::Task::GenericStackTask::MigrateSecureStoreTask;

use strict;
use warnings;
use parent 'LCM::Task';
use LCM::Task::TaskProgressHandler;
use SDB::Install::SysVars qw($path_separator);
use SDB::Install::Globals qw(
    $gSecureStoreTypeLSS
    $gOperationMigrateSecureStore
    isHostRoleLssCompatible
    GetLssCompatibleHostRoles
);

sub new {
    my ($class, $configuration, $target) = @_;
    my $self = $class->SUPER::new($configuration);
    $self->setTarget($target);
    return $self;
}

sub getId {
    return 'migrate_secure_store';
}

sub getName{
    my ($self) = @_;
    return "Migrate secure store to ".$self->getTarget();
}

sub getExecutionName{
    my ($self) = @_;
    return "Migrate secure store to ".$self->getTarget();
}

sub _isLocalhostLssCompatible {
    my ($self, $instance) = @_;
    my $localhost = $instance->get_host();
    my $roles = $instance->getHostRolesByIniFile($localhost);
    foreach my $role (@{$roles}) {
        return 1 if isHostRoleLssCompatible($role);
    }
    return 0;
}

sub _findLssCompatibleHost {
    my ($self, $instance) = @_;
    my $lssHosts = $instance->getHostsByRole(GetLssCompatibleHostRoles());
    if (defined $lssHosts && @{$lssHosts} >= 1) {
        return @$lssHosts[0];
    }
    return undef;
}

sub _executeInternal {
    my ($self) = @_;
    my $msg = $self->getMsgLst()->addProgressMessage($self->getExecutionName());
    my $saveCtx = $self->setMsgLstContext([$msg->getSubMsgLst(), $msg->getSubMsgLst()]);

    my $rc = $self->_migrateSecureStore();
    my $status = $self->getStatus();
    my $state = $rc ? $status->_setFinishedState() : $status->_setErrorState();

    $self->setMsgLstContext($saveCtx);
    return $rc;
}

sub _migrateSecureStore {
    my ($self) = @_;
    my $configuration = $self->_getConfiguration();
    my $instance = $configuration->getOwnInstance(1);
    if (!defined $instance) {
        $self->getErrMsgLst()->addError("Failed to get HDB instance");
        return undef;
    }

    if ($self->_isLocalhostLssCompatible($instance)) {
        return $instance->migrateSecureStore($self->getTarget(), $configuration, $self->getMsgLst());
    } else {
        my $lssHost = $self->_findLssCompatibleHost($instance);
        if (! defined $lssHost) {
            $self->setErrorMessage("Failed to find LSS-compatible host.", $self->getErrMsgLst());
            return undef;
        }
        $self->getMsgLst()->addMessage("Found LSS-compatible host ($lssHost) to migrate secure store to ".$self->getTarget());
        return $self->_migrateSecureStoreRemotely($instance, $configuration, $lssHost, $self->getTarget());
    }
}

sub _migrateSecureStoreRemotely {
    my ($self, $instance, $config, $lssHost, $secureStoreType) = @_;

    my $rHosts = $config->getRemoteHosts();

    $self->getMsgLst()->addProgressMessage("Migrating Secure Store...");

    my $progress_message = "Migrating Secure Store...";
    my $done_message     = "Migration of Secure Store done.";
    my $error_message    = "Migration of Secure Store failed!";

    # ex. /hana/shared
    my $sapmnt = $config->getSAPSystem()->getHanaInstPath();

    # ex. /usr/sap/<SID>/HDB<NR>
    my $path = $instance->get_instanceDir();

    my $target = $secureStoreType eq $gSecureStoreTypeLSS ? 'LSS' : 'SSFS';

    my $rc = 0;
    if ($rHosts->isHostctrl) {
        $rc = $self->_executeSHAOperation(
            $progress_message, $done_message, $error_message,
            $sapmnt, $instance, $path, $target,
            $rHosts, $lssHost, $config
        );
    } else {
        $rc = $self->_executeSSHOperation(
            $progress_message, $done_message, $error_message,
            $sapmnt, $instance, $path, $target,
            $rHosts, $lssHost, $config
        );
    }

    $self->getMsgLst()->appendMsgLst($rHosts->getMsgLst());

    if (defined($rc) && $rc != 0) {
        $instance->AddError("Migration of Secure Store failed!", $rHosts);
        return undef;
    }
    return 1;
}

sub _executeSHAOperation {
    my ($self,
        $progress_message, $done_message, $error_message,
        $sapmnt, $instance, $path, $target,
        $rHosts, $lssHost, $config) = @_;

    my $optionMap = {
        'SAPMNT'   => $sapmnt,
        'SID'      => $instance->get_sid(),
        'PATH'     => $path,
        'TARGET'   => $target,
    };
    my $actionID  = $config->{options}->{action_id};
    $optionMap->{ACTION_ID} = $actionID if (defined $actionID);

    my $rc = $rHosts->executeHostctrlParallel($gOperationMigrateSecureStore,
                                            $config,
                                            undef,     # parameter IDs
                                            undef,     # password keys
                                            undef,     # opt ignore
                                            undef,     # opt timeout
                                            $progress_message,
                                            $done_message,
                                            $error_message,
                                            $optionMap,
                                            undef,     # host map option map
                                            [$lssHost]  # execute only hosts
                                        );
    return $rc;
}

sub _executeSSHOperation {
    my ($self,
        $progress_message, $done_message, $error_message,
        $sapmnt, $instance, $path, $target,
        $rHosts, $lssHost, $config) = @_;

    my $hdblcmexec = $config->getResidentInstallerExecutable();
    my $cmd = "$hdblcmexec -main SDB::Install::App::Console::HdbMigrateSecureStore::main" .
                           ' --sid='    . $instance->get_sid() .
                           ' --path='   . $path .
                           ' --target=' . $target;
    my $rc = $rHosts->executeParallel(
                        $cmd,
                        $progress_message,
                        $done_message,
                        $error_message,
                        undef,           # Password input stream
                        undef,           # Template values
                        [$lssHost]        # Array of hostnames, on which to execute the command
                    );
    return $rc;
}

sub _getNumberOfExpectedOutputLines{
    return 2;
}

sub shouldBeShownOnSummaryPage {
    return 0;
}

sub initProgressHandler {
    my ($self) = @_;
    $self->getMsgLst()->setProgressHandler(LCM::Task::TaskProgressHandler->new($self));
}

sub setTarget {
    my ($self, $target) = @_;
    $self->{target} = $target;
}

sub getTarget {
    my ($self) = @_;
    return $self->{target};
}

1;
