package LCM::Task::RenameTask::RenameSystemTask;

use strict;
use parent qw(LCM::Task::SHASupportedTask);

use LCM::Component;
use LCM::ShellConfigurator;
use LCM::SapHostAgentFunctions;
use LCM::Utils::CommonUtils qw(buildMapParamOptions);
use File::Basename qw (basename);
use LCM::Landscape::LandscapeDescription;
use LCM::Python::SLDReconfigurationHelper;
use SDB::Install::SysVars qw($path_separator $isWin);
use SDB::Install::Globals qw($gLogDir $gProductNameSHA);
use LCM::TraceLoggingEnvironment;
use LCM::Installer;

my $PROGRESS_MESSAGE = 'Renaming System';
our @EXPORT = qw ($PROGRESS_MESSAGE);

my $ext = $isWin ? '.exe' : '';

# Override
sub getId {
    return 'rename_system';
}

# Override
sub getName {
    return "Rename System";
}

# Override
sub getExecutionName {
    return $PROGRESS_MESSAGE;
}

# Override
sub _getNumberOfExpectedOutputLines {
    return 100;
}

# Override
sub _executeInternal {
    my $self = shift();
    my $configuration = $self->_getConfiguration();
    my $msg = $self->getMsgLst ()->addProgressMessage ($self->getExecutionName(). "...");
    my $saveCntxt = $self->setMsgLstContext([$msg->getSubMsgLst()]);
    my $progressHandler = $self->getMsgLst()->getProgressHandler();
    my $shouldIndent = defined($progressHandler) && $progressHandler->can('incrementIndentationDepth');
    $progressHandler->incrementIndentationDepth() if $shouldIndent;

    my $exitCode = $self->_renameSystem($configuration);
    if(!$exitCode){
        $self->getStatus()->_setErrorState();
        $self->setErrorMessage(sprintf('%s failed', $self->getName()), $self->getErrMsgLst());      
    } else {
        $self->getStatus()->_setFinishedState();
    }

    my $endMessage = defined($exitCode) ? 'Renaming system' : 'Renaming system failed';
    $msg ->endMessage (undef, $endMessage);
    $self->setMsgLstContext($saveCntxt);
    $progressHandler->decrementIndentationDepth() if $shouldIndent;
    return;
}

sub _renameSystem {
    my ( $self, $configuration ) = @_;
    my $isContinueAction   = $configuration->{isContinueOption};
    my $hdbRenameArguments = $self->_buildArguments ( $configuration );
    my $hdbRenameXmlInput  = $configuration->getXmlPasswordStream();
    if (defined $hdbRenameXmlInput) {
        push( @{$hdbRenameArguments}, '--read_password_from_stdin=xml' );
    }
    push( @{$hdbRenameArguments}, '--batch' );
    push( @{$hdbRenameArguments}, '--instlog_dir=' . $gLogDir);

    return undef if !$self->_executeHdbRename ( $configuration, $hdbRenameArguments, $hdbRenameXmlInput);
    $self->configureRenamedEnv($configuration);
    return undef if (!$isContinueAction && $configuration->isSIDChanged() && !$self->_correctComponentLmStructureLocation($configuration));
    if ( $configuration->shouldRunHdbRename() ) {
        return undef if !$self->_updateSystemState($configuration);
        return undef if (!$isContinueAction && $configuration->isSIDChanged() && !$self->_reconfigureSLD($configuration));
    }
    return 1;
}

sub getProgressHandler {
    return $_[0]->getMsgLst()->getProgressHandler();    
}

sub _correctComponentLmStructureLocation {
    my ( $self, $configuration) = @_;

    $self->getMsgLst()->AddMessage('Correcting Component List location...');
    my $newSID = $configuration->{params}->{newSID}->{value};
    my $lmstructurePath = join( $path_separator, 'usr', 'sap', $newSID, 'lm_structure' );

    if ( -e $lmstructurePath && -l $lmstructurePath )    # if exists and is a symbolic link
    {
        $self->getMsgLst()->AddMessage('Changing location of the Component List file...');
        $self->getMsgLst()->AddMessage("Unlinking $lmstructurePath...");
        my $rc = unlink($lmstructurePath);
        return undef unless $rc;

        my $sharedSidLmStructurepath = $configuration->getSharedNewSIDPath() . $path_separator . 'lm_structure';
        $self->getMsgLst()->AddMessage("Linking $lmstructurePath to $sharedSidLmStructurepath...");
        my $rc1 = symlink( $sharedSidLmStructurepath, $lmstructurePath );
        return undef unless $rc1;
        return 1;
    } 

    $self->getMsgLst()->AddMessage('Component List file does not need correction.');
    return 1;
}

sub _updateSystemState {
    my ($self, $configuration) = @_;
    my $isContinueAction = $configuration->{isContinueOption};

    if ( !$isContinueAction ) {
        my $sid = $configuration->getValue('newSID') || $configuration->getValue('SID');
        my $progressMessage = 'Updating Component List...';
        my $failedMessage = 'Updating Component List failed';
        my $message = $self->getMsgLst()->addProgressMessage($progressMessage);
        my $landscapeDescription = new LCM::Landscape::LandscapeDescription([$message->getSubMsgLst()], $sid);

        if(!$landscapeDescription->getErrMsgLst()->isEmpty()){
            my $errorMessage = 'Update of Component List failed';
            $self->getErrMsgLst()->addError($errorMessage, $landscapeDescription->getErrMsgLst());
            $message->endMessage($failedMessage);
            return undef;
        }
        if($landscapeDescription->store()){
            $message->endMessage();
            return 1;
        }
        $message->endMessage($failedMessage);
        return undef;
    }

    if ( !$isWin && $configuration->isSIDChanged() ) {
        $self->getMsgLst()->addMessage('Updating shell configurations...');
        my $homedir = $configuration->getSidAdmHomeDir();
        my $shellConfigurator = LCM::ShellConfigurator->new($homedir);

        return undef unless $shellConfigurator->updateBashrcFile();
        return undef unless $shellConfigurator->updateBashSapenvFile();
        return undef unless $shellConfigurator->updateCshrcFile();
        return undef unless $shellConfigurator->updateCshSapenvFile();

        # TODO: check if timezone must be upadted
    }
    return 1;
}

sub _reconfigureSLD {
    my ( $self, $configuration ) = @_;

    if ($isWin) {
        $self->getMsgLst()->addMessage('SLD configuration is currently not supported under Windows.');
        return 1;
    }

    $self->getMsgLst()->addMessage('Checking SLD configuration...');
    my $ownInstanceAfterRename = $configuration->getOwnInstanceAfterRename();    # use renamed system instance
    return undef unless $ownInstanceAfterRename;

    my $sldConfigurator = LCM::Python::SLDReconfigurationHelper->new($ownInstanceAfterRename);
    $sldConfigurator->setMsgLstContext( $self->getMsgLstContext() );

    my $newSID = $configuration->{params}->{newSID}->{value};
    my $isSldEnabled = $sldConfigurator->isSldEnabled( $newSID );
    return undef if not defined $isSldEnabled;

    if ($isSldEnabled) {
        my $sharedSidLmStructurepath = $configuration->getSharedNewSIDPath() . $path_separator . 'lm_structure';
        return undef
          unless $sldConfigurator->setLmStructurePath( $newSID, $sharedSidLmStructurepath);

        my $isInstanceRunning = $self->_isInstanceRunning($configuration);
        return undef if not defined $isInstanceRunning;

        if ($isInstanceRunning) {
            $self->getMsgLst()->addMessage("Configuring SLD...");


            my $exitCode = $sldConfigurator->reconfigureSLD( $newSID );

        }
        else {
            $self->getMsgLst()->addMessage("There is no running Instance Service found. Skipping SLD configuration.");
        }

    }
    else {
        $self->getMsgLst()->addMessage("SLD is not enabled for SID $newSID. Skipping SLD configuration.");
    }
    return 1;
}

sub _isInstanceRunning {
    my ($self, $configuration) = @_;

    my $hdbInstance = $configuration->getOwnInstanceAfterRename();
    if (!defined $hdbInstance) {
        return undef;
    }

    my $host = $hdbInstance->get_host();
    my $instanceNumber = $hdbInstance->get_nr();
    require SDB::Install::SAPControl;
    my $sapControl = new SDB::Install::SAPControl($host,
                                                  $instanceNumber,
                                                  undef, # user
                                                  undef, # password
                                                  $configuration->isUseHttps(),
                                                  $configuration->getValue ('SSOCertificate'));

    return $sapControl->isRunning();
}

sub _buildArguments {
    my ( $self, $configuration ) = @_;

    if ($configuration->{isContinueOption}) {
        return $self->_buildScopeIndependentArguments($configuration);
    }

    my $paramIdsToSkip = ['HostMap', 'TenantMap', 'Scope', 'RootUser', 'InternalNetwork'];

    # TODO: check what should be added for Windows
    if ($isWin) {
        push(@{$paramIdsToSkip}, 'HomeDir', 'Shell', 'UID', 'GID');
    }

    my $arguments = $configuration->getCmdLineArgsFromInstconfig(new SDB::Install::Configuration::Rename(),
        $paramIdsToSkip, 'Rename', 'hdbrename');

    if ( $configuration->isAnyHostChanged () && $configuration->hasValue('HostMap')) {
        my $hostMapArgs = buildMapParamOptions($configuration, 'HostMap', 1);
        push @{$arguments}, @{$hostMapArgs};
    }

    if ($configuration->hasValue('TenantMap')) {
        my $tenantMapArgs = buildMapParamOptions($configuration, 'TenantMap', 1);
        push @{$arguments}, @{$tenantMapArgs};
    }

    my $internalNetwork = $configuration->getValue('InternalNetwork');
    if(defined($internalNetwork) && !$configuration->isSkipped('InternalNetwork') && length($internalNetwork) > 0 && $internalNetwork ne 'none') {
        push(@$arguments, "--internal_network=$internalNetwork");
    }

    if (!$isWin && $configuration->isDistributedSystem(1) )
    {
        # currently not supported for Windows
        my $scopeValue = $configuration->getValue('Scope');

        if (!defined $scopeValue || ($scopeValue ne 'instance')) {
            if ($configuration->hasValue('RootUser')) {
                push( @{$arguments}, $configuration->getOpt('RootUser')
                                . '='  . $configuration->{params}->{RootUser}->{value} );
            }
            $scopeValue = 'system';
        }
        push @{$arguments}, $configuration->getOpt('Scope') . '='  . $scopeValue;
    }
    if ($configuration->isRegisterScenario()) {
        my $shouldUseSystemd = $configuration->getValue('UseSystemd');
        push(@$arguments, "--is_register=1", "--use_systemd=$shouldUseSystemd");
    }
    if ($configuration->getValue('IsRegisterLSS')) {
        push(@$arguments, "--is_register_lss=1");
    }
    return $arguments;
}

sub _buildScopeIndependentArguments {
    my ($self, $configuration) = @_;
    my $configurationParamIds = $configuration->getParamIds();
    my $scopeIndependentParamIds = ['NoStart', 'KeepUser', 'KeepUserHomeDir', 'InitUser', 'InitUserHomeDir', 'AutoStart',
                                    'XSSpaceUserIdProd', 'XSSpaceUserIdSAP', 'XsEaDataPath', 'IsRegisterLSS', 'UsePMem',
                                    'PMemMountPoints', 'BasePathPMem'];
    my $scopeDependentParamIds = _removeElementsFromArray($configurationParamIds, $scopeIndependentParamIds);
    return $configuration->getCmdLineArgsFromInstconfig(new SDB::Install::Configuration::Rename(),
        $scopeDependentParamIds, 'Rename', 'hdbrename');
}

sub _removeElementsFromArray {
    my ($array, $elementsToRemove) = @_;
    my $modifiedArray = $array;

    foreach my $elementToRemove (@{$elementsToRemove}) {
        my $index=0;
        $index++ until @$modifiedArray[$index] eq $elementToRemove;
        splice(@$modifiedArray, $index, 1);
    }

    return $modifiedArray;
}

sub _getHdbRenameExecutable {
    my ( $self, $configuration ) = @_;

    my $sharedSidPath = $configuration->getHanaSharedSidFromHdblcmRuntimeDir();

    my $hdbRenameExecutable =
        join( $path_separator, $sharedSidPath, 'global', 'hdb', 'install', 'bin', 'hdbrename' )
        . $ext;

    return $hdbRenameExecutable;
}

sub _executeHdbRename {
    my ( $self, $configuration, $hdbRenameArguments, $hdbRenameXmlInput ) = @_;
    my $hdbRenameExecutable = $self->_getHdbRenameExecutable ( $configuration );
    my $exer = LCM::ProcessExecutor->new ( $hdbRenameExecutable, $hdbRenameArguments,
        $hdbRenameXmlInput );
     $exer->setMsgLstContext( $self->getMsgLstContext() );
    $exer->setOutputHandler($self->getMsgLst()->getProgressHandler());
    $exer->setProcessEnvironment(LCM::TraceLoggingEnvironment::PrepareComponentHdbInstallerEnvironment(1,'server'));
    my $rc = $exer->executeProgram ( );
    if(!defined $rc || $rc)
    {
        my $flavourProductName = $configuration->getFlavourProductName();
        $self->setErrorMessage ( "Error occurred while executing $flavourProductName System rename (hdbrename)", $exer->genHdbErrMsgLstFromOutput());
        $rc = undef;
       
    } else {
        $rc = 1;
    }
    return wantarray ? ($rc, $exer) : $rc;
}

sub configureRenamedEnv {
    my ($self, $config) = @_;
    return 1 if(!$config->isSIDChanged());

    my $installer = LCM::Installer->new();
    $installer->setMsgLstContext($config->getMsgLstContext());
    return $installer->renameHanaSID($config->getValue('SID'), $config->getValue('newSID'), $config->getValue('Target'));
}

1;
