package LCM::Utils::CommonUtils;

use strict;
use parent 'Exporter';
use SDB::Install::SAPSystem;
use SDB::Install::Globals qw($gDirNameCockpitStack);
use SDB::Install::System qw(isRegularFile);
use LCM::Manifests::CockpitStackManifest;
use LCM::Installer;
use experimental qw(smartmatch);
use File::stat;
use File::Spec;
use File::Basename qw(dirname);

our @EXPORT_OK = qw(
	isSingleDB
	getSidadmName
	getSystemSidadmUid
	getSystemSidadmGid
	isSystemReplication
	findAutoConfiguration
	getLocalHostSidadmUid
    getLocalHostSidadmGid
	getSpecialOptionString
	removeElementFromArray
	getHdbmodifyTimeoutValues
	getXSInstallerTimeoutValues
	getHdblcmUploadDirectory
	getCockpitProductVersion
    buildTimeoutsString
    buildMapParamOptions
);

sub getHdblcmUploadDirectory {
    require File::Spec;

    my ($sid) = @_;
    return File::Spec->catfile('/usr', 'sap', $sid, 'hdblcm_uploads', 'SAPHostAgentUploads');
}

sub findAutoConfiguration {
    my ($action, $flavour) = @_;
    $action //= 'install';

    my $installer = new LCM::Installer();
    my $runtimeDir = $installer->getRuntimeDir();
    return undef if !$runtimeDir || !File::stat::stat($runtimeDir);

    my @candidateFilenames = ("auto_$action.cfg");
    unshift (@candidateFilenames, "auto_${action}_".lc($flavour).".cfg") if $flavour; # The flavoured auto-config has higher priority

    my @candidatePaths = map { File::Spec->catfile(dirname($runtimeDir), 'configurations', $_) } @candidateFilenames;
    my @existingCandidates = grep { isRegularFile($_) } @candidatePaths;
    return @existingCandidates ? $existingCandidates[0] : undef;
}

sub getSpecialOptionString {
    my ($sourceOption, $targetOption) = @_;
    my $optionArguments = defined($sourceOption) && defined($targetOption) ? $sourceOption->getSetKeysValidFor($targetOption) : undef;
    my $optionString = defined($optionArguments) ? $sourceOption->getOptionStringFromArgList($optionArguments) : undef;

    return $optionString;
}

sub removeElementFromArray {
	my ($array, $removeElement) = @_;

	if (not ($removeElement ~~ @$array)) {
		return $array;
	}

	my $index=0;
	$index++ until @$array[$index] eq $removeElement;
	splice(@$array, $index, 1);

	return $array;
}

sub getHdbmodifyTimeoutValues {
	return ['start_instance', 'start_service']; # Should be updated if hdbmodify changes its timeout support
}

sub getXSInstallerTimeoutValues {
	return ['start_instance', 'start_service', 'stop_instance', 'stop_service']; # Should be updated if hdbinst of XSA changes its timeout support
}

sub buildTimeoutsString {
    my ($config, $timeoutValues, $withOption) = @_;
    my $timeoutOption = $config->getOptionTimeout();
    my $timeoutString = $timeoutOption ? $timeoutOption->getOptionStringFromArgList($timeoutValues) : undef;
    if ($timeoutString) {
        return ($withOption // 1) ? $timeoutString : (split('=', $timeoutString))[1];
    }
    return undef;
}

sub buildMapParamOptions {
    my ($config, $paramID, $ignoreEqualKeyValues) = @_;
    my $opt = $config->getOpt($paramID);
    my $mapValue = $config->getValue($paramID);
    my @mapKeys = keys %{$mapValue};
    if ($ignoreEqualKeyValues) {
        @mapKeys = grep { $_ ne $mapValue->{$_} } @mapKeys;
    }
    return [ map { "$opt=$_=$mapValue->{$_}" } @mapKeys ];
}

sub getSidadmName {
	my ($configuration) = @_;
	return undef if ( ! defined $configuration->getValue('SID'));
	return lc($configuration->getValue('SID')) . "adm";
}

sub getSystemSidadmUid {
	my ($configuration) = @_;
	my $sapSystem = $configuration->getSAPSystem();
    my $userConfigurationFile = (defined $sapSystem) ? $sapSystem->getUserConfig() : undef;
	if (!defined $userConfigurationFile){
		$configuration->setErrorMessage(undef, $sapSystem->getErrMsgLst()) if defined ($sapSystem);
		return undef;
	}
	return $userConfigurationFile->{sidadm_id};
}

sub getSystemSidadmGid {
	my ($configuration) = @_;
	my $sapSystem = $configuration->getSAPSystem();
	my $userConfigurationFile = $sapSystem->getUserConfig();
	if (!defined $userConfigurationFile){
		$configuration->setErrorMessage( undef, $sapSystem->getErrMsgLst() );
		return undef;
	}
	return $userConfigurationFile->{sapsys_groupid};
}

sub getLocalHostSidadmUid {
	my ($configuration) = @_;
	my $sidadmName = getSidadmName($configuration);
	return undef if ( ! defined $sidadmName);
	my $localHostSidadmUid = getpwnam($sidadmName);
	return $localHostSidadmUid;
}

sub getLocalHostSidadmGid {
    my ($configuration) = @_;
    my $sidadmName = getSidadmName($configuration);
    return undef if ( ! defined $sidadmName);
    my (undef, undef, undef, $localHostSidadmGid) = getpwnam($sidadmName);
    return $localHostSidadmGid;
}

sub isSingleDB {
    require LCM::Installer;
    my $installer  = new LCM::Installer();
    my $sid        = $installer->getSid();
    my $allSystems = CollectSAPSystems( undef, 1 );
    my $sapSys     = (defined $sid) ? $allSystems->{$sid} : undef;
    my $instance   = (defined $sapSys && $sapSys->hasNewDB())
                     ? $sapSys->getNewDBInstances()->[0] : undef;
    my $dbMode     = (defined $instance) ? $instance->getDbMode() : undef;
    my $isSingleDb = ( defined $dbMode && $dbMode eq 'singledb' );

    return $isSingleDb;
}

sub isSystemReplication {
    my ($configuration) = @_;
    my $instance   = $configuration->getOwnInstance();
    $instance->setMsgLstContext($configuration->getMsgLstContext()) if(defined $instance);
    return (defined $instance) ? $instance->useSystemReplication(1) : 0;

}

sub isSAPEnvironment {
    require Net::Ping;

    my $url = 'portal.wdf.sap.corp';
    my $p;
    my $result = 0; # false by default
    eval{
         $p = Net::Ping->new("external");
    };
    if ($@){
        # write message in trace file
        require SDB::Install::MsgLst;
        new SDB::Install::MsgLst ()->addError ("Cannot ping $url: $@");
        return $result;
    }
# Since its last version (2.55), Net::Ping::ping started croaking whenever the host cannot be resolved
# The eval block prevents hdblcm from exiting with unhandled exception in such cases
    local $SIG{__WARN__} = sub {};
    eval {
        $result = $p->ping($url, 2) ? 1 : 0;
    };
    $p->close();
    return $result;
};

sub getCockpitProductVersion {
	my ($sapSys) = @_;
	my $manifestDir = File::Spec->catdir($sapSys->get_globalSidDir(), $gDirNameCockpitStack);
	my $manifestPath = File::Spec->catfile($manifestDir, 'manifest');
	return $sapSys->GetVersion() if(!File::stat::stat($manifestPath));
	my $manifest = LCM::Manifests::CockpitStackManifest->new($manifestPath);
	my $cockpitVersion = $manifest->getVersion();
	return (defined $cockpitVersion)? $cockpitVersion : $sapSys->GetVersion();
}

1;