package SDB::Install::Configuration::PMemConfig;

use strict;
use parent qw(SDB::Install::Configuration::AnyConfig);

use SDB::Common::BuiltIn;
use SDB::Common::ArrayUtils qw(hasDuplicates);
use SDB::Install::SAPSystem qw($STEP_CREATE_INSTANCE);
use SDB::Install::System qw(getMountPoint
                            isDirectory
                            listDir);

use IO::Dir;
use File::stat;
use File::Spec;


sub getParamUsePMem {
    my ($self, $order, $section) = @_;

    return {
        'order'             => $order,
        'opt'               => 'use_pmem',
        'type'              => 'boolean',
        'section'           => $section,
        'value'             => undef,
        'default'           => 0,
        'str'               => 'Enable usage of persistent memory',
        'init_with_default' => 1,
        'set_interactive'   => 0,
        'skip'              => 0,
        'persStep'          => $STEP_CREATE_INSTANCE
    };
}

sub getParamBasePathPMem {
    my ($self, $order, $section) = @_;
    my $str = 'Location of persistent memory volumes';
    return {
        'order'                     => $order,
        'opt'                       => 'pmempath',
        'type'                      => 'path',
        'opt_arg'                   => '<path>',
        'section'                   => $section,
        'value'                     => undef,
        'default'                   => undef,
        'str'                       => $str,
        'interactive_str'           => $str,
        'init_with_default'         => 1,
        'set_interactive'           => 1,
        'skip'                      => 1,
        'persStep'                  => $STEP_CREATE_INSTANCE,
        'custom_input'              => \&SDB::Install::Configuration::PMemConfig::readInteractiveBasePathPMem
    };
}

sub getParamPMemMountPoints {
    my ($self, $order, $section) = @_;
    return {
        'order'                     => $order,
        'opt'                       => 'pmem_mountpoints',
        'type'                      => 'csv',
        'opt_arg'                   => '<mountpoint1>[,mountpoint2]...',
        'section'                   => $section,
        'value'                     => undef,
        'default'                   => undef,
        'str'                       => 'Locations of persistent memory mount points',
        'interactive_str'           => 'comma-separated locations of persistent memory mount points',
        'init_with_default'         => 1,
        'set_interactive'           => 1,
        'skip'                      => 1,
        'persStep'                  => $STEP_CREATE_INSTANCE,
        'custom_input'              => \&SDB::Install::Configuration::PMemConfig::readInteractivePMemMountPoints
    };
}

sub readInteractiveInput {
    my ($config, $paramId) = @_;
    my $builtIn = SDB::Common::BuiltIn->get_instance();
    my $str = $config->getInteractiveString($paramId);
    my $default = $config->getDefault($paramId);
    my $input;
    while (1) {
        $builtIn->print("Enter $str" . (defined $default ? " [$default]: " : ": "));
        chomp($input = <STDIN>);
        next if (!$input && !$default);
        return $input;
    }
}

sub readInteractivePMemMountPoints {
    my ($console, $config) = @_;
    return readInteractiveInput($config, 'PMemMountPoints');
}

sub readInteractiveBasePathPMem {
    my ($console, $config) = @_;
    return readInteractiveInput($config, 'BasePathPMem');
}

sub checkUsePMem {
    my ($self, $value) = @_;

    if (!$value) {
        $self->setSkip('BasePathPMem', 1);
        $self->setSkip('PMemMountPoints', 1);
        return 1;
    }

    my $param = $self->{params}->{UsePMem};
    my $errorMessage = "Error checking $param->{str}: ";
    my $rc = 1;

    if (!$self->collectPMemFilesystems()){
        my $ignore = $self->getIgnore('check_pmem') // 0;
        return $self->_handlePmemError("${errorMessage}Cannot detect PMEM filesystems on the host", $ignore) if !$ignore;
    }
    return $self->_handlePmemParametersDependency($errorMessage);
}

sub _handlePmemParametersDependency {
    my ($self, $errorMessage) = @_;
    my $pmemPathOption = $self->getOpt('BasePathPMem');
    my $pmemMountPointsOption = $self->getOpt('PMemMountPoints');
    if ($self->getBatchValue('BasePathPMem') && $self->getBatchValue('PMemMountPoints')) {
        return $self->_handlePmemError("${errorMessage}${pmemMountPointsOption} and $pmemPathOption cannot be used together.", 0);
    }
    if ($self->getBatchValue('BasePathPMem')) {
        $self->setSkip('BasePathPMem', 0);
    } else {
        $self->setSkip('PMemMountPoints', 0);
    }
    return 1;
}

sub collectPMemFilesystems {
    my ($self) = @_;
    my $mounts = new SDB::Install::System::Mounts();
    my $nvmMounts = $mounts->getNVMMounts();
    my $tmpfsMounts = $mounts->getTmpfsMounts();

    if ((!defined $nvmMounts || @$nvmMounts == 0) &&
        (!defined $tmpfsMounts || @$tmpfsMounts == 0)) {
        return 0;
    }
    $self->{_nvmMounts} = $nvmMounts;
    $self->{_tmpfsMounts} = $tmpfsMounts;
    return 1;
}

sub checkBasePathPMem {
    my ($self, $value) = @_;
    my $ignore = $self->getIgnore('check_pmem') // 0;
    my $paramStr = $self->{params}->{BasePathPMem}->{str};
    my $errorMessage = "Error checking $paramStr: ";
    if (!isDirectory($value)) {
        return $self->_handlePmemError("${errorMessage}Cannot access directory '$value': $!", $ignore);
    }
    my $contents = listDir($value);
    if (@$contents == 0) {
        return $self->_handlePmemError("${errorMessage}No pmem mount points found in '$value'", $ignore);
    }
    my @mountPoints = map { File::Spec->catfile($value, $_) } @$contents;

    return $self->verifyPMemConfig(\@mountPoints, $errorMessage, $ignore)
           && $self->verifyRemoteHostsPMemConfig($paramStr);
}

sub checkPMemMountPoints {
    my ($self, $value) = @_;
    my $ignore = $self->getIgnore('check_pmem') // 0;
    my $param = $self->{params}->{PMemMountPoints};
    my $paramStr = $param->{str};
    my $errorMessage = "Error checking $paramStr: ";
    my @mountPoints = split(',', $value);
    if (hasDuplicates(\@mountPoints)) {
        return $self->_handlePmemError("${errorMessage}The input value contains duplicated mount points.", 0);
    }

    return $self->verifyPMemConfig(\@mountPoints, $errorMessage, $ignore)
           && $self->verifyRemoteHostsPMemConfig($paramStr);
}

sub verifyPMemConfig {
    my ($self, $mountPoints, $errorMessage, $ignore) = @_;
    $errorMessage //= '';
    if (!$self->collectPMemFilesystems()) {
        return $self->_handlePmemError("${errorMessage}Cannot detect PMEM filesystems on the host.", $ignore);
    }
    return $self->_handlePmemError("${errorMessage}No pmem mount points specified.", $ignore) if (!@$mountPoints);

    my $rc = 1;
    my @pmemLst;
    foreach my $volDir (@$mountPoints){
        if (!isDirectory($volDir)) {
            $rc &= $self->_handlePmemError("${errorMessage}Cannot access '$volDir': $!", $ignore);
            next;
        }
        my ($mountPoint, $success, $error) = getMountPoint($volDir);
        if (!$self->_checkMount($mountPoint, $self->{_nvmMounts}) && !$self->_checkMount($mountPoint, $self->{_tmpfsMounts})) {
            $rc &= $self->_handlePmemError("${errorMessage}Directory '$volDir' is not mounted as DAX or tmpfs filesystem", $ignore);
            next;
        }
        push @pmemLst, $volDir;
    }
    $self->setPMemMountpoitsList(\@pmemLst) if (@pmemLst);
    return $rc;
}

sub verifyRemoteHostsPMemConfig {
    my ($self, $paramStr) = @_;
    return 1 if (!$self->getRemoteHosts());
    my $collectOnlyMap = { pmemMountPoints => 1 };
    $self->SDB::Install::Configuration::AnyConfig::CollectOtherHostInfos(1, undef, undef, $collectOnlyMap);

    if (!exists $self->{pmemMountPointsVerifiedHostmap}) {
        my $hostnames = $self->getRemoteHosts()->getHostNames();
        $self->addWarning("$paramStr cannot be verified on hosts: ".join(', ', @$hostnames).". Make sure they're configured correctly.");
        return 1;
    }

    my $hostmapHash = $self->{pmemMountPointsVerifiedHostmap};
    my $ignore = $self->getIgnore('check_pmem') // 0;

    my $rc = 1;
    foreach my $host (keys %$hostmapHash) {
        if (!$hostmapHash->{$host}->{isVerified}) {
            $rc &= $self->_handlePmemError("$paramStr not configured correctly on host $host.", $ignore, $hostmapHash->{$host}->{msgLst});
        }
    }
    return $rc;
}

sub _checkMount {
    my ($self, $path, $mountedVolumes) = @_;
    if (!defined $mountedVolumes) {
        return 0;
    }
    return grep { $path eq $_->{mnt_dir} } @$mountedVolumes;
}

sub _handlePmemError {
    my ($self, $errorMessage, $ignore, $subMsgLst) = @_;
    my $msg;
    if ($ignore) {
        $msg = $self->addWarning($errorMessage);
        $msg->getSubMsgLst()->setMsgLst($subMsgLst) if defined ($subMsgLst);
        $self->getMsgLst()->addMessage('Ignoring error');
        return 1;
    }
    $msg = $self->getMsgLst()->addError($errorMessage);
    $self->getErrMsgLst()->appendMsg($msg);
    $msg->getSubMsgLst()->setMsgLst($subMsgLst) if defined ($subMsgLst);
    return 0;
}

1;
