package SDB::Install::FastRestartOptionConfigurator;

use strict;
use warnings;
use parent 'SDB::Install::Base';
use LCM::FileUtils qw(listDirectory createDirectory changeFileOwnership changeFilePermissions);
use SDB::Install::LayeredConfig qw(CFG_LAYER_HOST CFG_LAYER_SYSTEM);
use SDB::Install::System qw(isRegularFile isDirectory which);

sub new {
    my ($class, $config) = @_;
    my $self = $class->SUPER::new();
    $self->setConfiguration($config);
    return $self;
}

sub configureHost {
    my ($self, $trexInstance) = @_;
    my $message = "Configuring fast restart option";
    my $msg = $self->getMsgLst()->addProgressMessage("$message ...");
    my $saveContext = $self->setMsgLstContext([$msg->getSubMsgLst(), $msg->getSubMsgLst()]);

    my $rc = $self->createTmpfsDirectories();
    $rc &&= $self->setupTmpfsMounts();
    $rc &&= $self->adaptFstabConfigFile();
    $rc &&= $self->updateGlobalIni($trexInstance);

    $message .= (!$rc) ? ' failed' : ' finished';
    $msg->endMessage(undef, $message);
    $self->setMsgLstContext($saveContext);
    return $rc;
}

sub getNumaNodes {
    my ($self) = @_;
    return $self->{numaNodes} if defined $self->{numaNodes};

    my $nodeRegex = qr/^node([0-9]+)$/;
    my $numaNodesRoot = File::Spec->catfile('', 'sys', 'devices', 'system', 'node');
    my @nodes = grep { $_ =~ $nodeRegex} @{listDirectory($numaNodesRoot) // []};
    my @nodeNumbers = ();
    foreach my $node (@nodes) {
        my $nodePath = File::Spec->catfile($numaNodesRoot, $node);
        next if !isDirectory($nodePath);
        push @nodeNumbers, ($node =~ $nodeRegex);
    }
    $self->{numaNodes} = \@nodeNumbers;
    return $self->{numaNodes};
}

sub createTmpfsDirectories {
    my ($self) = @_;
    my $configuration = $self->getConfiguration();
    my $sapsys = $configuration->getSAPSystem();
    my $uid = $configuration->getValue('UID') // $sapsys->getUID();
    my $gid = $configuration->getValue('GID') //$sapsys->getGID();
    my $errMsgLst = $self->getErrMsgLst();
    my $msgLst = $self->getMsgLst();
    my $permissions = 0777;
    my $tmpfsDir;
    my $rc = 1;

    foreach my $node (@{$self->getNumaNodes()}) {
        $tmpfsDir = $self->getTmpfsMountpointPerNode($node);
        $msgLst->addMessage("Creating directory $tmpfsDir...");
        if(isDirectory($tmpfsDir)){
            $msgLst->addMessage("Directory $tmpfsDir already exists");
            $msgLst->addMessage("Adjust ownership and permissions (uid = $uid, gid = $gid, permissions = 0777)");
            $rc &&= changeFileOwnership($tmpfsDir, $uid, $gid, $errMsgLst);
            $rc &&= changeFilePermissions($tmpfsDir, $permissions, $errMsgLst);
        } else {
            $rc &&= createDirectory($tmpfsDir, $permissions, $uid, $gid, 1, $errMsgLst);
        }
    }
    return $rc;
}

sub setupTmpfsMounts {
    my ($self) = @_;
    my $sid = $self->getConfiguration()->getSID();
    my ($tmpfsDir, $args, $executor);
    $self->getMsgLst()->addMessage("Mounting persistent memory volumes...");
    my $mountPath = which('mount');

    foreach my $node (@{$self->getNumaNodes()}) {
        $tmpfsDir = $self->getTmpfsMountpointPerNode($node);
        $args = ['tmpfs'.$sid.$node, '-t', 'tmpfs', '-o', 'mpol=prefer:'.$node, $tmpfsDir];
        $executor = LCM::ProcessExecutor->new($mountPath, $args);
        $executor->setMsgLstContext($self->getMsgLstContext());
        my $exitCode = $executor->executeProgram();
        return undef if(!defined $exitCode || $exitCode);
    }
    return 1;
}

sub getTmpfsMountpointPerNode {
    my ($self, $node) = @_;
    my $configuration = $self->getConfiguration();
    my $path = $configuration->getValue('TmpfsMountpointsPath');
    my $sid = $configuration->getSID();
    return File::Spec->catfile($path, 'tmpfs'.$node, $configuration->getSID());
}

sub adaptFstabConfigFile {
    my ($self) = @_;
    my $file = File::Spec->catfile('', 'etc', 'fstab');
    my $fileHandle = undef;

    $self->getMsgLst()->addMessage("Updating $file...");
    if(!open($fileHandle, '>>',  $file)){
        $self->appendErrorMessage("Failed to open file '$file' for writing: $!");
        return undef;
    }

    my $sid = $self->getConfiguration()->getSID();
    my ($tmpfsDir, $device, $line);
    foreach my $node (@{$self->getNumaNodes()}) {
        $tmpfsDir = $self->getTmpfsMountpointPerNode($node);
        $device = 'tmpfs'.$sid.$node;
        $line = "$device $tmpfsDir tmpfs rw,relatime,mpol=prefer:$node 0 0\n";
        $self->getMsgLst()->addMessage("Adding line $line");
        if(! print $fileHandle $line) {
            $self->appendErrorMessage("Cannot write into file  '$file': $!");
            return undef;
        }
    }

    if(! close($fileHandle)) {
        $self->appendErrorMessage("Cannot close file '$file': $!");
        return undef;
    }

    return 1;
}

sub updateGlobalIni{
    my ($self, $trexInstance) = @_;
    my $config = $self->getConfiguration();
    my $instance = $config->getOwnInstance() // $trexInstance;
    my $globalIni = $instance->getGlobalIni();
    my $msgLst = $self->getMsgLst();

    if (!defined $globalIni) {
        $msgLst->addError('Cannot access global.ini');
        return undef;
    }

    $msgLst->addMessage('Updating global.ini...');
    $globalIni->setMsgLstContext($self->getMsgLstContext());
    my @volumes = map { $self->getTmpfsMountpointPerNode($_) } @{$self->getNumaNodes()};
    $globalIni->setValue(CFG_LAYER_HOST, 'persistent_memory', 'basepath_persistent_memory_volumes', join(';', @volumes));
    $globalIni->setValue(CFG_LAYER_HOST, 'memorymanager', 'persistent_memory_global_allocation_limit', $config->getValue('PmemGlobalAllocationLimit'));

    foreach my $param ('enable_compaction','allow_tmpfs_block_deletion_for_shrink', 'sr_allow_tmpfs_block_deletion_for_shrink') {
        $globalIni->setValue(CFG_LAYER_SYSTEM, 'persistent_memory', $param, 'true');
    }

    if (!defined $globalIni->write()) {
        $msgLst->addError('Cannot write to global.ini');
        return undef;
    }
    return 1;
}

sub getConfiguration{
    my ($self) = @_;
    return $self->{_config};
}

sub setConfiguration{
    my ($self, $config) = @_;
    $self->{_config} = $config;
}

1;
