package SDB::Install::OSUserGroupOperationsBase;

use strict;
use warnings;

use Storable;
use SDB::Install::MsgLst;
use SDB::Common::BuiltIn;

my $RETRIES_BEFORE_FAILURE = 10;
my $DELAY_BETWEEN_RETRIES = 2;

sub _executeOsUserGroupOperationWithRetries {
    my ($self, $cmd, $args, $referenceToMsgLstOrCfg) = @_;
    my ($rc, $lastIterationMsgLstOrCfg);
    for(my $retriesRemaining = $RETRIES_BEFORE_FAILURE; $retriesRemaining > 0; $retriesRemaining--) {
        # deep cloning of the MsgLst/config hash is needed so that when the execution fails and another one is started, it will only display the messages from the last execution
        $lastIterationMsgLstOrCfg = $self->_cloneMsgLstOrCfg(${$referenceToMsgLstOrCfg});
        my @copiedArgsArray = @{$args}; # needed because exec_program modifies the args array and this leads to worsening the situation with each iteration
        $rc = SDB::Install::System::exec_program($cmd, \@copiedArgsArray, $lastIterationMsgLstOrCfg);
        last if $rc == 0;
        $self->_sleepBeforeNextAttempt();
    }
    if($rc != 0) {
        SDB::Install::MsgLst::addWarning($lastIterationMsgLstOrCfg, "Execution of \"$cmd\" program failed after $RETRIES_BEFORE_FAILURE retries.");
    }
    # after we're done retrying, we need to swap the original object with the cloned one from the last attempt
    ${$referenceToMsgLstOrCfg} = $lastIterationMsgLstOrCfg;
    return $rc;
}

sub _executeAddUserToGroupWithRetries {
    my ($self, $username, $group, $referenceToMsgLstOrCfg) = @_;
    my ($rc, $lastIterationMsgLstOrCfg);
    for(my $retriesRemaining = $RETRIES_BEFORE_FAILURE; $retriesRemaining > 0; $retriesRemaining--) {
        # deep cloning of the MsgLst/config hash is needed so that when the execution fails and another one is started, it will only display the messages from the last execution
        $lastIterationMsgLstOrCfg = $self->_cloneMsgLstOrCfg(${$referenceToMsgLstOrCfg});
        $rc = $self->_executeAddUserToGroup($username, $group, $lastIterationMsgLstOrCfg);
        last if $rc == 0;
        $self->_sleepBeforeNextAttempt();
    }
    if($rc != 0) {
        SDB::Install::MsgLst::addWarning($lastIterationMsgLstOrCfg, "Adding user \"$username\" to group \"$group\" failed after $RETRIES_BEFORE_FAILURE retries.");
    }
    # after we're done retrying, we need to swap the original object with the cloned one from the last attempt
    ${$referenceToMsgLstOrCfg} = $lastIterationMsgLstOrCfg;
    return $rc;
}

sub _cloneMsgLstOrCfg {
    my ($self, $msgLstOrCfg) = @_;

    my $fLogSubstExists = exists $msgLstOrCfg->{fLogSubst};
    my $fLogSubst = $msgLstOrCfg->{fLogSubst}; # dclone doesn't support code ref cloning, so fLogSubst code ref needs to be saved somewhere and then 
    delete $msgLstOrCfg->{fLogSubst} if $fLogSubstExists;

    my $clonedMsgLstOrCfg = Storable::dclone($msgLstOrCfg);
    if($fLogSubstExists) {
        $clonedMsgLstOrCfg->{fLogSubst} = $fLogSubst;
        $msgLstOrCfg->{fLogSubst} = $fLogSubst; # needs to be restored on the OG object as well or next iterations using this object won't be able to use fLogSubst
        # local variables cannot be used as well as this is a reference to a blessed object
    }

    return $clonedMsgLstOrCfg;
}

sub _sleepBeforeNextAttempt {
    my $builtIn = SDB::Common::BuiltIn->get_instance();
    $builtIn->sleep($DELAY_BETWEEN_RETRIES);
    return 1;
}

sub _executeAddUserToGroup {
    my ($self, $username, $group, $msgLst) = @_;
    my $cmd = 'groupmod';
    my @args = ('-A', $username, $group);

    my $addUserToGroupMsgLst = SDB::Install::MsgLst->new();
    local %ENV = %ENV;
    $ENV{'PATH'} = '/sbin:/usr/sbin:' . $ENV{PATH};
    my $rc = SDB::Install::System::exec_program($cmd, \@args, $addUserToGroupMsgLst);

    if ($rc == 2){

        # "groupmod -A" (SuSE syntax) returned "wrong usage"
        # falling back to redhat/debian syntax "usermod -a -G"

        $cmd = 'usermod';
        @args = ('-a', '-G', $group, $username);
        $addUserToGroupMsgLst = new SDB::Install::MsgLst ();
        $rc = SDB::Install::System::exec_program($cmd, \@args, $addUserToGroupMsgLst);
    }

    $msgLst->appendMsgLst($addUserToGroupMsgLst);

    return $rc;
}