package SDB::Install::Utils::ParameterMigration::GlobalIniParameterMigrator;

use SDB::Install::LayeredConfig;
use SDB::Install::Utils::ParameterMigration::GlobalIniMigratableParameters;

use strict;

sub new {
    my ($class, $trexInstance) = @_;
    my $self = {
        ownInstance => $trexInstance,
        migratableParameters => SDB::Install::Utils::ParameterMigration::GlobalIniMigratableParameters->new(),
    };

    bless $self, $class;
}

sub migrateGlobalIniParamsFromDefaultToSystemLayer {
    my ($self) = @_;
    my $paramsToMigrate = $self->_getMigratableParametersList();
    foreach my $param (@{$paramsToMigrate}){
        my $shouldMigrateParameter = $self->_checkIfParameterShouldBeMigrated($param);
        next if !$shouldMigrateParameter;
        $self->_migrateParameterValueIfPossible($param);
    }
    return 1;
}

sub _checkIfParameterShouldBeMigrated {
    my ($self, $migratableParam) = @_;
    my $checkMethodName = "_shouldMigrate${migratableParam}ParameterValue";
    my $checkMethodSub = $self->can($checkMethodName);
    if($checkMethodSub){
        return &$checkMethodSub($self);
    }
    return 1;
}

sub _migrateParameterValueIfPossible {
    my ($self, $param) = @_;
    my $migratableParam = $self->_getMigratableParameters()->getParameter($param);
    my @sections = @{$migratableParam->{'sections'}};
    my $paramKey = $migratableParam->{'iniKey'};

    my $progressMsg = "Migrating $paramKey from default to system layer global.ini if possible";
    my $instance = $self->_getOwnInstance();
    my $msg = $instance->getMsgLst()->addMessage("$progressMsg");
    my $subMsgLst = $msg->getSubMsgLst();

    my $globalIni = $self->_getGlobalIni();

    if (!defined $globalIni){
        $subMsgLst->addError('Cannot access global.ini');
        return 0;
    }

    $globalIni->setMsgLstContext([$subMsgLst]);

    foreach my $section (@sections) {
        $self->_migrateParameterFromDefaultToSystemLayer($section, $paramKey, $subMsgLst);
    }

    if (!defined $globalIni->write()){
        $subMsgLst->addError('Cannot write to global.ini');
        return 0;
    }
    return 1;
}

sub _migrateParameterFromDefaultToSystemLayer {
    my ($self, $section, $paramKey, $msglst) = @_;
    my $globalIni = $self->_getGlobalIni();
    my $instance = $self->_getOwnInstance();

    my $paramDefaultLayerValue = $globalIni->getValue($section, $paramKey, CFG_LAYER_DEFAULT, 1);
    my $paramSystemLayerValue = $globalIni->getValue($section, $paramKey, CFG_LAYER_SYSTEM, 1);

    if (!defined $msglst) {
        $msglst = $instance->getMsgLst();
    }

    if(!defined $paramDefaultLayerValue || defined $paramSystemLayerValue) {
        $msglst->addMessage("Will not migrate $paramKey [$section] from default to system layer global.ini; either $paramKey [$section] is not defined in the default layer or it is already defined in the system layer");
        return 0;
    }

    $globalIni->setValue(CFG_LAYER_SYSTEM, $section, $paramKey, $paramDefaultLayerValue);
    $msglst->addMessage("Successfully migrated $paramKey [$section] from default to system layer global.ini");
    return 1;
}

sub _shouldMigrateEnableSslParameterValue {
    my ($self) = @_;
    my $globalIni = $self->_getGlobalIni();
    my $enableSslParam = $self->_getMigratableParameters()->getParamEnableSsl();
    my @sections = @{$enableSslParam->{'sections'}};
    my $paramKey = $enableSslParam->{'iniKey'};

    foreach my $section (@sections) {
        my $paramDefaultLayerValue = $globalIni->getValue($section, $paramKey, CFG_LAYER_DEFAULT, 1);
        return 0 if($paramDefaultLayerValue eq 'on');
    }
    return 1;
}

sub _getOwnInstance {
    my ($self) = @_;
    return $self->{ownInstance};
}

sub _getGlobalIni {
    my ($self) = @_;
    my $ownInstance = $self->_getOwnInstance();
    my $globalIni = $ownInstance->getGlobalIni();
    return $globalIni;
}

sub _getMigratableParameters {
    my ($self) = @_;
    return $self->{migratableParameters};
}

sub _getMigratableParametersList {
    my ($self) = @_;
    return $self->_getMigratableParameters()->getParametersList();
}

1;