package driver

/*
#cgo CFLAGS: -I${SRCDIR}/includes
#cgo LDFLAGS: -Wl,-rpath,\$ORIGIN
#include <stddef.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "DBCAPI_DLL.h"
#include "DBCAPI.h"
*/
import "C"
import (
	"container/list"
	"context"
	"database/sql"
	"database/sql/driver"
	"errors"
	"fmt"
	"io"
	"math"
	"reflect"
	"strconv"
	"time"
	"unsafe"
)

//DriverName is the driver name to use with sql.Open for hdb databases.
const DriverName = "hdb"

//For conversion between string and timestamp
const TS_LAYOUT_STRING = "2006-01-02 15:04:05.000000000"

//For conversion between string and timestamp without without seconds
const SECONDDATE_LAYOUT_STRING = "2006-01-02 15:04:05"

//For conversion between string and date
const DATE_LAYOUT_STRING = "2006-01-02"

//For conversion between string and time
const TIME_LAYOUT_STRING = "15:04:05"

//Max length of varbinary column
const MAX_BINARY_SIZE = 5000

const DEFAULT_ISOLATION_LEVEL = 1

const EXPECTED_DBCAPI_VERSION = "libdbcapiHDB 2.17.022.1689711322"

const (
	FunctionCode_DDL    = 1
	FunctionCode_INSERT = 2
	FunctionCode_UPDATE = 3
	FunctionCode_DELETE = 4
)

var init_ok bool

type drv struct{}

//database connection
type connection struct {
	dbcapi_conn *C.dbcapi_connection
	dsnInfo *DsnInfo
}

type transaction struct {
	dbcapi_conn *C.dbcapi_connection
}

//statement
type statement struct {
	query               string
	dbcapi_stmt         *C.dbcapi_stmt
	conn                *connection
	Ok                  bool
	rowsetFetchSize     uint
	functionCode        int
	boundParameterInfos *paramInfoValues // Used in executeBatchInternal
	err                 error            // Permanent error for the statement (e.g. not being able to get param information required for execution)
	// Rows are first track in a map (in increasing order of rows), then for a specific row
	// the WriteLOBs are stored (with knowledge of the column index), in increasing order of columns.
	// Both indices are zero based
	storedWriteLobs     map[uint]*list.List
	fetchUsingReadLob   bool
}

const (
	// strconv.IntSize gives the size (in bits) of an int/uint - this is also changes with the size
	// of pointers, depending on the build type (32 or 64 bit OS). See the following links for more
	// information:
	// 1) https://golang.org/pkg/strconv/ (strconv package (IntSize constant))
	// 2) https://stackoverflow.com/questions/25741841/how-can-i-check-if-my-program-is-compiled-for-32-or-64-bit-processor
	pointerSize = C.size_t(strconv.IntSize / 8)
	nullSize    = C.size_t(C.sizeof_dbcapi_bool)
	boolSize    = C.size_t(C.sizeof_dbcapi_bool)
	lengthSize  = C.size_t(C.sizeof_size_t)
)

//result set
type rows struct {
	dbcapi_stmt       *C.dbcapi_stmt
	conn              *connection
	rowsetFetchSize   uint
	boundColumns      *C.dbcapi_data_value
	boundColumnInfo   *C.dbcapi_column_info
	boundColumnsCount uint
	isFromStatement   bool
	isClosed          bool
	err               error
	fetchUsingReadLob bool
}

func (rows *rows) cleanup() {
	// boundColumnsCount should only be zero if the column binding
	// in bindColumnArraysForBulkFetch() was not done or resulted in an error
	// (a table cannot have 0 columns)
	if rows.rowsetFetchSize == 0 || rows.boundColumnsCount == 0 {
		return
	}

	// Remaining pointers are set to nil here for the case of
	// usage in Multiple Result Sets
	if rows.boundColumnInfo != nil {
		C.free(unsafe.Pointer(rows.boundColumnInfo))
		rows.boundColumnInfo = nil
	}

	if rows.boundColumns != nil {
		for i := 0; uint(i) < rows.boundColumnsCount; i++ {
			boundColumn := (*C.struct_dbcapi_data_value)(
				unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumns)) + uintptr(C.sizeof_struct_dbcapi_data_value*C.size_t(i))))
			if boundColumn.is_null != nil {
				C.free(unsafe.Pointer(boundColumn.is_null))
			}

			if boundColumn.length != nil {
				C.free(unsafe.Pointer(boundColumn.length))
			}

			if boundColumn.buffer != nil {
				C.free(unsafe.Pointer(boundColumn.buffer))
			}
		}
		C.free(unsafe.Pointer(rows.boundColumns))
		rows.boundColumns = nil
	}
	rows.boundColumnsCount = 0
}

//keep track of rows affected after inserts and updates
type result struct {
	rowsAffected int64
	err          error
}

//needed to handle nil time values
type NullTime struct {
	Time  time.Time
	Valid bool
}

//needed to handle nil binary values
type NullBytes struct {
	Bytes []byte
	Valid bool
}

type BulkNullInt64 struct {
	Int64s       []int64
	Valids       []bool
	NumberOfRows uint64
}

type BulkNullFloat64 struct {
	Float64s     []float64
	Valids       []bool
	NumberOfRows uint64
}

type BulkNullBool struct {
	Bools        []bool
	Valids       []bool
	NumberOfRows uint64
}

type BulkNullString struct {
	Strings      []string
	Valids       []bool
	NumberOfRows uint64
}

type BulkNullBytes struct {
	Bytes        [][]byte
	Valids       []bool
	NumberOfRows uint64
}

type BulkNullTime struct {
	Times        []time.Time
	Valids       []bool
	NumberOfRows uint64
}

// Needed to handle bound parameters
type boundParameterValue struct {
	boundStruct *C.struct_dbcapi_bind_data
	isBatch     bool
	batchSize   uint64
}

func (bpv *boundParameterValue) cleanup() {
	if bpv.boundStruct != nil {
		defer C.free(unsafe.Pointer(bpv.boundStruct))

		if bpv.isBatch {
			bpv.cleanupBatch()
		} else {
			bpv.cleanupNonBatch()
		}
	}
}

func (bpv *boundParameterValue) cleanupNonBatch() {
	if bpv.boundStruct.value.is_null != nil {
		C.free(unsafe.Pointer(bpv.boundStruct.value.is_null))
	}

	if bpv.boundStruct.value.buffer != nil {
		C.free(unsafe.Pointer(bpv.boundStruct.value.buffer))
	}

	if bpv.boundStruct.value.length != nil {
		C.free(unsafe.Pointer(bpv.boundStruct.value.length))
	}
}

func (bpv *boundParameterValue) cleanupBatch() {
	if bpv.boundStruct.value.is_null != nil {
		C.free(unsafe.Pointer(bpv.boundStruct.value.is_null))
	}

	if bpv.boundStruct.value.length != nil {
		C.free(unsafe.Pointer(bpv.boundStruct.value.length))
	}

	if bpv.boundStruct.value.buffer != nil {
		defer C.free(unsafe.Pointer(bpv.boundStruct.value.buffer))

		for i := uint64(0); i < bpv.batchSize; i++ {
			valuePtr := (**C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(bpv.boundStruct.value.buffer)) +
				uintptr(pointerSize*C.size_t(i))))
			if *valuePtr != nil {
				C.free(unsafe.Pointer(*valuePtr))
			}
		}
	}
}

// Needed to handle dbcapi_bind_param_info struct deallocations (only done for columns of one row)
type paramInfoValues struct {
	paramInfos []*C.dbcapi_bind_param_info
}

func (bpi *paramInfoValues) cleanup() {
	if bpi.paramInfos != nil {
		// Assumes that all locations have a value associated
		for i := 0; i < len(bpi.paramInfos); i++ {
			if bpi.paramInfos[i] != nil {
				C.free(unsafe.Pointer(bpi.paramInfos[i]))
			}
		}
	}
}

func init() {
	sql.Register(DriverName, &drv{})
	cName := C.CString("hdbGolangDriver")
	defer C.free(unsafe.Pointer(cName))
	api := C.dbcapi_init(cName, C.DBCAPI_API_VERSION_1, nil)
	if api != 1 {
		init_ok = false
	} else {
		init_ok = true
	}
}

func getError(dbcapi_conn *C.dbcapi_connection) error {
	if dbcapi_conn == nil {
		return driver.ErrBadConn
	}

	errorLen := C.dbcapi_error_length(dbcapi_conn)
	if errorLen == 0 {
		errorLen = 1
	}
	err := make([]byte, errorLen)
	errCode := C.dbcapi_error(dbcapi_conn, (*C.char)(unsafe.Pointer(&err[0])), errorLen)
	//-10806: connection lost and transaction rolled back and -10807: connection lost and -10108: reconnect error and -10821 session not connected
	if errCode == -10806 || errCode == -10807 || errCode == -10108 || errCode == -10821 {
		return driver.ErrBadConn
	}

	if int(errCode) != 0 {
		return fmt.Errorf("%d: %s", errCode, string(err[:errorLen-1]))
	}

	return nil
}

func isBadConnection(dbcapi_conn *C.dbcapi_connection) bool {
	if dbcapi_conn == nil {
		return true
	}

	len := C.size_t(C.DBCAPI_ERROR_SIZE)
	err := make([]byte, len)
	errCode := C.dbcapi_error(dbcapi_conn, (*C.char)(unsafe.Pointer(&err[0])), len)
	//-10806: connection lost and transaction rolled back and -10807: connection lost and -10108: reconnect error and -10821 session not connected
	return errCode == -10806 || errCode == -10807 || errCode == -10108 || errCode == -10821
}

func setPropertiesAndConnect(dbcapi_conn *C.dbcapi_connection, dsnInfo *DsnInfo) error {
	err := setDefaultConnectProperties(dbcapi_conn, dsnInfo)
	if err != nil {
		return err
	}

	err = setAdditionalConnectProperties(dbcapi_conn, dsnInfo)
	if err != nil {
		return err
	}

	connRes := C.dbcapi_connect2(dbcapi_conn)

	if connRes != 1 {
		return getError(dbcapi_conn)
	}
	return nil
}

func setDefaultConnectProperties(dbcapi_conn *C.dbcapi_connection, dsnInfo *DsnInfo) error {
	{
		cServernodeKey := C.CString("SERVERNODE")
		defer C.free(unsafe.Pointer(cServernodeKey))
		cServernode := C.CString(dsnInfo.Host)
		defer C.free(unsafe.Pointer(cServernode))
		if C.dbcapi_set_connect_property(dbcapi_conn, cServernodeKey, cServernode) != 1 {
			return getError(dbcapi_conn)
		}
	}
	{
		cUsernameKey := C.CString("UID")
		defer C.free(unsafe.Pointer(cUsernameKey))
		cUsername := C.CString(dsnInfo.Username)
		defer C.free(unsafe.Pointer(cUsername))
		if C.dbcapi_set_connect_property(dbcapi_conn, cUsernameKey, cUsername) != 1 {
			return getError(dbcapi_conn)
		}
	}
	{
		cPasswordKey := C.CString("PWD")
		defer C.free(unsafe.Pointer(cPasswordKey))
		cPassword := C.CString(dsnInfo.Password)
		defer C.free(unsafe.Pointer(cPassword))
		if C.dbcapi_set_connect_property(dbcapi_conn, cPasswordKey, cPassword) != 1 {
			return getError(dbcapi_conn)
		}
	}
	{
		cCharsetKey := C.CString("CHARSET")
		defer C.free(unsafe.Pointer(cCharsetKey))
		cCharset := C.CString("UTF-8")
		defer C.free(unsafe.Pointer(cCharset))
		if C.dbcapi_set_connect_property(dbcapi_conn, cCharsetKey, cCharset) != 1 {
			return getError(dbcapi_conn)
		}
	}
	return nil
}

func setAdditionalConnectProperties(dbcapi_conn *C.dbcapi_connection, dsnInfo *DsnInfo) error {
	connProps := dsnInfo.ConnectProps
	for key := range connProps {
		values := connProps[key]
		cKey := C.CString(key)
		defer C.free(unsafe.Pointer(cKey))
		cValue := C.CString(values[0])
		defer C.free(unsafe.Pointer(cValue))
		if C.dbcapi_set_connect_property(dbcapi_conn, cKey, cValue) != 1 {
			return getError(dbcapi_conn)
		}
	}
	return nil
}

func (d *drv) Open(dsn string) (driver.Conn, error) {
	if !init_ok {
		return nil, fmt.Errorf("Loading the dbcapi library has failed, please ensure that it is on your library path")
	}

	buffer := (*C.char)(C.malloc(256))
	defer C.free(unsafe.Pointer(buffer))
	client := C.dbcapi_client_version(buffer, 256)
	version := C.GoString(buffer)

	if client != 1 {
		return nil, fmt.Errorf("Unable to retrieve the version string from libdbcapiHDB.")
	} else if version != EXPECTED_DBCAPI_VERSION {
		return nil, fmt.Errorf("Invalid libdbcapiHDB version.  Expected '%s', but found '%s'.", EXPECTED_DBCAPI_VERSION, version)
	}

	dsnInfo, err := parseDSN(dsn)
	if err != nil {
		return nil, err
	}

	dbcapi_conn := C.dbcapi_new_connection()
	if dbcapi_conn == nil {
		return nil, errors.New("dbcapi failed to allocate a connection object")
	}

	err = setPropertiesAndConnect(dbcapi_conn, dsnInfo)

	if err != nil {
		C.dbcapi_free_connection(dbcapi_conn)
		return nil, err
	}

	autoCommRes := C.dbcapi_set_autocommit(dbcapi_conn, 1)

	if autoCommRes != 1 {
		defer C.dbcapi_free_connection(dbcapi_conn)
		return nil, getError(dbcapi_conn)
	}
	return &connection{dbcapi_conn: dbcapi_conn, dsnInfo: dsnInfo}, nil
}

// CheckNamedValue (from the NamedValueChecker interface) is implemented here to allow our
// custom types implementation
func (statement *statement) CheckNamedValue(nv *driver.NamedValue) error {
	if nv != nil && statement.dbcapi_stmt != nil {
		if statement.functionCode == 0 {
			statement.functionCode = int(C.dbcapi_get_function_code(statement.dbcapi_stmt))
		}

		v := reflect.ValueOf(nv.Value)

		if v.IsValid() {
			if statement.functionCode == FunctionCode_INSERT ||
				statement.functionCode == FunctionCode_UPDATE ||
				statement.functionCode == FunctionCode_DELETE {
				// Special Accepted Type for Bulk Insert/Update/Delete
				if _, ok := v.Interface().([][]interface{}); ok {
					return nil
				// Special Accepted Type for WriteLOB streaming
				} else if _, ok := v.Interface().(*WriteLOB); ok {
					return nil
				}
			}
		}
	}
	return driver.ErrSkip
}

func (connection *connection) Prepare(query string) (driver.Stmt, error) {
	psql := C.CString(query)
	defer C.free(unsafe.Pointer(psql))
	ps := C.dbcapi_prepare(connection.dbcapi_conn, psql)

	if ps == nil {
		return nil, getError(connection.dbcapi_conn)
	}

	return &statement{query: query, dbcapi_stmt: ps, conn: connection,
		storedWriteLobs: make(map[uint]*list.List)}, nil
}

func (connection *connection) Close() error {
	defer C.dbcapi_free_connection(connection.dbcapi_conn)
	disconn := C.dbcapi_disconnect(connection.dbcapi_conn)
	var err error
	if disconn != 1 {
		err = getError(connection.dbcapi_conn)
	}
	return err
}

func (connection *connection) Begin() (driver.Tx, error) {
	autoCommRes := C.dbcapi_set_autocommit(connection.dbcapi_conn, 0)
	if autoCommRes != 1 {
		return nil, getError(connection.dbcapi_conn)
	}

	return &transaction{dbcapi_conn: connection.dbcapi_conn}, nil
}

func (connection *connection) BeginTx(ctx context.Context, opts driver.TxOptions) (driver.Tx, error) {
	if opts.ReadOnly {
		return nil, errors.New("Setting of read only option is currently unsupported")
	}

	isolationLevel := int(opts.Isolation)
	//0 means that use the driver's default isolation level
	if isolationLevel == 0 {
		return connection.Begin()
	}
	if isolationLevel < 0 || isolationLevel > 3 {
		return nil, errors.New(fmt.Sprintf("Unsupported isolation level requested: %d ", isolationLevel))
	}
	err := setIsolationLevel(connection.dbcapi_conn, isolationLevel)
	if err != nil {
		return nil, err
	}

	return connection.Begin()
}

func setIsolationLevel(dbcapi_conn *C.dbcapi_connection, isolationLevel int) error {
	isolationLevelRes := C.dbcapi_set_transaction_isolation(dbcapi_conn, C.dbcapi_u32(isolationLevel))
	if isolationLevelRes != 1 {
		return getError(dbcapi_conn)
	}

	return nil
}

func (connection *connection) Exec(query string, args []driver.Value) (driver.Result, error) {
	if args != nil && len(args) != 0 {
		return nil, driver.ErrSkip
	}

	sql := C.CString(query)
	defer C.free(unsafe.Pointer(sql))
	dbcapi_stmt := C.dbcapi_execute_direct(connection.dbcapi_conn, sql)

	if dbcapi_stmt == nil {
		return nil, getError(connection.dbcapi_conn)
	}
	defer C.dbcapi_free_stmt(dbcapi_stmt)

	funcCode := C.dbcapi_get_function_code(dbcapi_stmt)
	if funcCode == FunctionCode_DDL {
		return driver.ResultNoRows, nil
	}

	rowsAffected := C.dbcapi_affected_rows(dbcapi_stmt)
	if rowsAffected == -1 {
		return &result{rowsAffected: -1, err: getError(connection.dbcapi_conn)}, nil
	} else {
		return &result{rowsAffected: int64(rowsAffected), err: nil}, nil
	}
}

func (connection *connection) Query(query string, args []driver.Value) (driver.Rows, error) {
	if args != nil && len(args) != 0 {
		return nil, driver.ErrSkip
	}

	sql := C.CString(query)
	defer C.free(unsafe.Pointer(sql))
	dbcapi_stmt := C.dbcapi_execute_direct(connection.dbcapi_conn, sql)

	if dbcapi_stmt == nil {
		return nil, getError(connection.dbcapi_conn)
	}

	return &rows{dbcapi_stmt: dbcapi_stmt, conn: connection, isFromStatement: false, isClosed: false, err: nil}, nil
}

func (rows *rows) Close() error {
	if !rows.isClosed {
		if rows.rowsetFetchSize >= 1 {
			defer rows.cleanup()
		}

		if !rows.isFromStatement {
			defer C.dbcapi_free_stmt(rows.dbcapi_stmt)
		}
		rows.isClosed = true
		closeRowsRes := C.dbcapi_reset(rows.dbcapi_stmt)

		if closeRowsRes != 1 {
			rows.err = getError(rows.conn.dbcapi_conn)
		}
	}
	return rows.err
}

func (rows *rows) Columns() []string {
	colCount := int(C.dbcapi_num_cols(rows.dbcapi_stmt))
	if colCount == -1 {
		return nil
	}
	columnNames := make([]string, colCount)
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))

	for i := 0; i < colCount; i++ {
		colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(i), colInfo)
		columnNames[i] = ""
		if colinfoRes == 1 {
			columnNames[i] = C.GoString(colInfo.name)
		}
	}

	return columnNames
}

func (rows *rows) ColumnTypeDatabaseTypeName(index int) string {
	dbTypeName := ""
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)
	if colinfoRes == 1 {
		dbTypeName = getNativeType(int(colInfo.native_type))
	}

	return dbTypeName
}

func (rows *rows) ColumnTypeNullable(index int) (bool, bool) {
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)

	nullable := false
	ok := false
	if colinfoRes == 1 {
		ok = true
		if colInfo.nullable == 1 {
			nullable = true
		}
	}

	return nullable, ok
}

func (rows *rows) ColumnTypePrecisionScale(index int) (int64, int64, bool) {
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)

	precision := int64(0)
	scale := int64(0)
	ok := false
	if colinfoRes == 1 && colInfo.native_type == C.DT_DECIMAL {
		precision = int64(colInfo.precision)
		scale = int64(colInfo.scale)
		ok = true
	}

	return precision, scale, ok
}

func (rows *rows) ColumnTypeLength(index int) (int64, bool) {
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)

	length := int64(0)
	ok := false
	if colinfoRes == 1 {
		dbcapiType := colInfo._type
		if dbcapiType == C.A_BINARY {
			length = int64(colInfo.max_size)
			ok = true
		} else if dbcapiType == C.A_STRING {
			//date, time, timestamp and decimal types are represented using string since:
			//1. dbcapi does not support date, time and timestamp types.
			//2. Go 1.8 does not have any decimal types
			hanaType := colInfo.native_type
			if hanaType != C.DT_DATE && hanaType != C.DT_TIME && hanaType != C.DT_TIMESTAMP && hanaType != C.DT_DECIMAL &&
				hanaType != C.DT_DAYDATE && hanaType != C.DT_SECONDTIME &&
				hanaType != C.DT_LONGDATE && hanaType != C.DT_SECONDDATE {
				length = int64(colInfo.max_size)
				ok = true
			}
		}
	}

	return length, ok
}

func (rows *rows) ColumnTypeScanType(index int) reflect.Type {
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)
	defer C.free(unsafe.Pointer(colInfo))

	if colinfoRes != 1 {
		//This function does not allow you to return an error. Return string since
		//most types can be converted to/from string.
		return reflect.TypeOf(string(""))
	}

	dbcapiType := colInfo._type
	switch dbcapiType {
	case C.A_BINARY:
		return reflect.TypeOf([]byte{0})
	case C.A_STRING:
		hanaType := colInfo.native_type
		//date, time, timestamp and decimal types are represented using string since dbcapi
		//does not support date, time and timestamp types.
		if hanaType == C.DT_DATE || hanaType == C.DT_TIME || hanaType == C.DT_TIMESTAMP ||
			hanaType == C.DT_DAYDATE || hanaType == C.DT_SECONDTIME ||
			hanaType == C.DT_LONGDATE || hanaType == C.DT_SECONDDATE {
			return reflect.TypeOf(time.Time(time.Time{}))
		}
		return reflect.TypeOf(string(""))
	case C.A_DOUBLE:
		return reflect.TypeOf(float64(0.0))
	case C.A_VAL64:
		return reflect.TypeOf(int64(0))
	case C.A_UVAL64:
		return reflect.TypeOf(uint64(0))
	case C.A_VAL32:
		return reflect.TypeOf(int32(0))
	case C.A_UVAL32:
		return reflect.TypeOf(uint32(0))
	case C.A_VAL16:
		return reflect.TypeOf(int16(0))
	case C.A_UVAL16:
		return reflect.TypeOf(uint16(0))
	case C.A_VAL8:
		return reflect.TypeOf(int8(0))
	case C.A_UVAL8:
		return reflect.TypeOf(uint8(0))
	case C.A_FLOAT:
		return reflect.TypeOf(float32(0.0))
	default:
		return reflect.TypeOf(string(""))
	}
}

func getNativeType(nativeTypeCode int) string {
	switch nativeTypeCode {
	case C.DT_DATE:
		return "DATE"
	case C.DT_TIME:
		return "TIME"
	case C.DT_TIMESTAMP:
		return "TIMESTAMP"
	case C.DT_DAYDATE:
		return "DATE"
	case C.DT_SECONDTIME:
		return "TIME"
	case C.DT_LONGDATE:
		return "TIMESTAMP"
	case C.DT_SECONDDATE:
		return "TIMESTAMP"
	case C.DT_VARCHAR1:
		return "VARCHAR"
	case C.DT_VARCHAR2:
		return "VARCHAR"
	case C.DT_ALPHANUM:
		return "ALPHANUM"
	case C.DT_CLOB:
		return "CLOB"
	case C.DT_CHAR:
		return "CHAR"
	case C.DT_STRING:
		return "STRING"
	case C.DT_DOUBLE:
		return "DOUBLE"
	case C.DT_REAL:
		return "REAL"
	case C.DT_DECIMAL:
		return "DECIMAL"
	case C.DT_INT:
		return "INT"
	case C.DT_SMALLINT:
		return "SMALLINT"
	case C.DT_BINARY:
		return "BINARY"
	case C.DT_VARBINARY:
		return "VARBINARY"
	case C.DT_BSTRING:
		return "BSTRING"
	case C.DT_BLOB:
		return "BLOB"
	case C.DT_ST_GEOMETRY:
		return "ST_GEOMETRY"
	case C.DT_ST_POINT:
		return "ST_POINT"
	case C.DT_TINYINT:
		return "TINYINT"
	case C.DT_BIGINT:
		return "BIGINT"
	case C.DT_BOOLEAN:
		return "BOOLEAN"
	case C.DT_NSTRING:
		return "NSTRING"
	case C.DT_SHORTTEXT:
		return "SHORTTEXT"
	case C.DT_NCHAR:
		return "NCHAR"
	case C.DT_NVARCHAR:
		return "NVARCHAR"
	case C.DT_NCLOB:
		return "NCLOB"
	case C.DT_TEXT:
		return "TEXT"
	case C.DT_BINTEXT:
		return "BINTEXT"
	default:
		return "UNKNOWN"
	}
}

func (rows *rows) Next(dest []driver.Value) error {
	fetchNextRes := C.dbcapi_fetch_next(rows.dbcapi_stmt)
	if fetchNextRes != 1 {
		if isBadConnection(rows.conn.dbcapi_conn) {
			return driver.ErrBadConn
		}
		return io.EOF
	}

	if rows.rowsetFetchSize >= 1 {
		return rows.getNextRowDataBulk(dest)
	} else {
		return rows.getNextRowData(dest)
	}
}

func (rows *rows) HasNextResultSet() bool {
	return true
}

func (rows *rows) NextResultSet() error {
	nextRs := C.dbcapi_get_next_result(rows.dbcapi_stmt)

	if nextRs != 1 {
		return io.EOF
	}

	if rows.rowsetFetchSize >= 1 {
		// Cleanup and rebind using the new set of metadata
		rows.cleanup()
		err := rows.doSetupForFetching(true)

		if err != nil {
			return err
		}
	}

	return nil
}

func checkBulkFetchSupported(native_type C.dbcapi_native_type) error {
	if native_type == C.DT_CLOB ||
		native_type == C.DT_NCLOB ||
		native_type == C.DT_BLOB {
		return errors.New("LOB types are not support for Bulk Fetching")
	} else if native_type == C.DT_ST_GEOMETRY ||
		native_type == C.DT_ST_POINT {
		return errors.New("Spatial types are not support for Bulk Fetching")
	}
	return nil
}

func getColumnSize(columnInfo *C.dbcapi_column_info) uint64 {
	// Fixed columns (Integer and Float have specific sizes)
	// Decimal/Time columns have fixed sizes, but are pulled as ASCII
	// Non-fixed columns (NVARCHAR and [VAR]BINARY) have sizes dependent on the precision.
	// We will be fetching NVARCHAR data as CESU-8/UTF-8 data, so we need to account for
	// worse-case character expansion (multibyte chacters up to 4 bytes in length)
	if columnInfo.native_type == C.DT_NCHAR ||
		columnInfo.native_type == C.DT_NVARCHAR {
		return 4*(uint64(columnInfo.precision)) + 1 // UTF-8 worse case characters can be up to 4 bytes in representation
	}

	return uint64(columnInfo.precision)
}

// This needs to be called for every new resultset, and it is expected that
// cleanup() is called before this, otherwise there will be native memory leakage
func (rows *rows) doSetupForFetching(isBulkFetch bool) error {
	columnCount := int(C.dbcapi_num_cols(rows.dbcapi_stmt))

	if columnCount < 1 {
		err := getError(rows.conn.dbcapi_conn)

		if err != nil {
			return fmt.Errorf("Could not obtain the number of ResultSet columns for the executed Query: %v", err)
		}

		return errors.New("Could not obtain the number of ResultSet columns for executed Query")
	}

	rows.boundColumnsCount = uint(columnCount)
	if isBulkFetch {
		rows.boundColumns = (*C.struct_dbcapi_data_value)(C.malloc(C.sizeof_struct_dbcapi_data_value * C.size_t(columnCount)))
	}
	rows.boundColumnInfo = (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info * C.size_t(columnCount)))

	var finalError error

	// We do not short-circuit the loop here on error to make the logic of
	// rows.cleanup() easier
	for i := 0; i < columnCount; i++ {
		columnInfo := (*C.struct_dbcapi_column_info)(unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumnInfo)) +
			uintptr(C.sizeof_struct_dbcapi_column_info*C.size_t(i))))
		ok := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(i), columnInfo)
		var err error

		if ok != 1 {
			return getError(rows.conn.dbcapi_conn)
		} else if isBulkFetch {
			err = checkBulkFetchSupported(columnInfo.native_type)

			if err != nil && finalError == nil {
				finalError = err
			}
			boundColumn := (*C.struct_dbcapi_data_value)(unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumns)) +
				uintptr(C.sizeof_struct_dbcapi_data_value*C.size_t(i))))
			C.memset(unsafe.Pointer(boundColumn), C.int(0), C.sizeof_struct_dbcapi_data_value)
			boundColumn.is_null = (*C.dbcapi_bool)(C.malloc(nullSize * C.size_t(rows.rowsetFetchSize)))
			boundColumn.length = (*C.size_t)(C.malloc(C.sizeof_size_t * C.size_t(rows.rowsetFetchSize)))
			boundColumn._type, boundColumn.buffer_size, boundColumn.buffer, err =
				getColumnBufferAndTypeInfo(columnInfo, rows.rowsetFetchSize, i)
			if err != nil && finalError == nil {
				finalError = err
			}
			ok = C.dbcapi_bind_column(rows.dbcapi_stmt, C.dbcapi_u32(i), boundColumn)
		}
	}

	return finalError
}

func getColumnBufferAndTypeInfo(columnInfo *C.dbcapi_column_info,
	rowsetFetchSize uint, columnIndex int) (C.dbcapi_data_type, C.size_t, *C.char, error) {
	var (
		dataType C.dbcapi_data_type
		size     C.size_t
		buffer   *C.char
		err      error
	)

	// Fixed columns (Integer and Float have specific sizes)
	// Decimal/Time columns have fixed sizes, but are pulled as ASCII
	// Non-fixed columns (NVARCHAR and [VAR]BINARY) have sizes dependent on the precision.
	// We will be fetching NVARCHAR data as CESU-8/UTF-8 data, so we need to account for
	// worse-case character expansion (multibyte chacters up to 4 bytes in length)
	size = C.size_t(columnInfo.precision)

	if columnInfo.native_type == C.DT_TINYINT ||
		columnInfo.native_type == C.DT_SMALLINT ||
		columnInfo.native_type == C.DT_INT ||
		columnInfo.native_type == C.DT_BIGINT {
		size = C.sizeof_int64_t
		dataType = C.A_VAL64
		buffer = (*C.char)(C.malloc(C.sizeof_int64_t * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_DOUBLE ||
		columnInfo.native_type == C.DT_REAL {
		size = C.sizeof_double
		dataType = C.A_DOUBLE
		buffer = (*C.char)(C.malloc(C.sizeof_double * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_BOOLEAN {
		size = C.sizeof_uint8_t
		dataType = C.A_UVAL8
		buffer = (*C.char)(C.malloc(C.sizeof_uint8_t * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_TIMESTAMP || columnInfo.native_type == C.DT_LONGDATE ||
		columnInfo.native_type == C.DT_SECONDDATE {
		// HANA defines the precision of TIMESTAMP columns as 27, due to the fractional part
		// being considered to be 7 digits long. However, the string itself contains the full
		// 9 digits, so we add 2 here, and consider the NULL
		size = 30
		dataType = C.A_STRING
		buffer = (*C.char)(C.malloc(C.sizeof_char * C.size_t(size) * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_TIME || columnInfo.native_type == C.DT_SECONDTIME ||
		columnInfo.native_type == C.DT_DATE || columnInfo.native_type == C.DT_DAYDATE {
		size = C.size_t(columnInfo.precision) + 1 // include NULL
		dataType = C.A_STRING
		buffer = (*C.char)(C.malloc(C.sizeof_char * C.size_t(size) * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_NVARCHAR ||
		columnInfo.native_type == C.DT_VARCHAR1 ||
		columnInfo.native_type == C.DT_VARCHAR2 ||
		columnInfo.native_type == C.DT_NCHAR ||
		columnInfo.native_type == C.DT_CHAR {
		// We will be fetching NVARCHAR data as CESU-8/UTF-8 data, so we need to account for
		// worse-case character expansion (multibyte chacters up to 4 bytes in length)
		size = 4*C.size_t(columnInfo.precision) + 1 // UTF-8 worse case characters can be up to 4 bytes in representation + NULL
		dataType = C.A_STRING
		buffer = (*C.char)(C.malloc(C.sizeof_char * size * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_BINARY ||
		columnInfo.native_type == C.DT_VARBINARY {
		// We are not allowing BLOB bindings here currently.
		// However this description may come back for SPATIAL data
		if columnInfo.precision == 0 {
			size = 5000
		}
		dataType = C.A_BINARY
		buffer = (*C.char)(C.malloc(C.sizeof_char * size * C.size_t(rowsetFetchSize)))
	} else if columnInfo.native_type == C.DT_DECIMAL ||
		columnInfo.native_type == C.DT_FIXED16 {
		dataType = C.A_STRING
		// Max Sizes
		// Fixed16 = [opt sign][38 digits with optional decimal point] + NULL = 41 characters
		// DecFloat =
		// [opt sign][34 digits of significand with decimal point ]E[opt sign][4 digit exponent] + NULL
		// = 43 characters
		// Using the larger of the two
		size = 43
		buffer = (*C.char)(C.malloc(C.sizeof_char * size * C.size_t(rowsetFetchSize)))
	} else {
		err = fmt.Errorf("Column %d: Unsupported Native Type %d", columnIndex,
			int(columnInfo.native_type))
	}

	return dataType, size, buffer, err
}

func (rows *rows) getNextRowDataBulk(dest []driver.Value) error {
	fetchedRows := int(C.dbcapi_fetched_rows(rows.dbcapi_stmt))

	if fetchedRows == -1 {
		return getError(rows.conn.dbcapi_conn)
	}

	for i := 0; i < len(dest); i++ {
		err := rows.getDataBulk(&dest[i], i, fetchedRows)
		if err != nil {
			return err
		}
	}

	return nil
}

// This is a temporary struct used to pass bound pointer data to
// This is an internal type only meant to be use for bulk fetching
// and is used to pass bound pointer data to Scanner() custom types.
// The C memory for the bound pointer types are cleaned up in
// rows.cleanup()
// If regular values are used to fetch data when in bulk fetch mode, this will result in an error
type bulkValue struct {
	dataType        C.dbcapi_data_type
	nativeType      C.dbcapi_native_type
	nulls           *C.dbcapi_bool
	bulkValues      *C.char
	lengths         *C.size_t
	precision       uint
	rowsetFetchSize uint
	numberOfRows    uint
	isInteger       bool
	isFloat         bool
	isBoolean       bool
	isDecimal       bool
	isString        bool
	isBinary        bool
	isTime          bool
	isDate          bool
	isSecondDate    bool
	isTimestamp     bool
}

// This is a internally-used struct used to pass extra information for ReadLOB
// handling.
// Any C memory for the bound pointer types are cleaned up in
// rows.cleanup()
type readLobValue struct {
	rows *rows // The actual rows object it refers to (needed for the C layer statement usage)
	columnIndex uint
}

func (readLobValue *readLobValue) isNull() (bool, error) {
	dataInfo := (*C.struct_dbcapi_data_info)(C.calloc(C.size_t(1), C.sizeof_struct_dbcapi_data_info))
	defer C.free(unsafe.Pointer(dataInfo))
	ok := C.dbcapi_get_data_info(readLobValue.rows.dbcapi_stmt,
		C.dbcapi_u32(readLobValue.columnIndex), dataInfo)

	if int(ok) == 0 {
		return false, getError(readLobValue.rows.conn.dbcapi_conn)
	}

	return (dataInfo.is_null == C.dbcapi_bool(1)), nil
}

func (rows *rows) getDataBulk(val *driver.Value, index int, fetchedRows int) error {
	col := (*C.struct_dbcapi_data_value)(unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumns)) +
		uintptr(C.sizeof_struct_dbcapi_data_value*C.size_t(index))))

	colInfo := (*C.struct_dbcapi_column_info)(unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumnInfo)) +
		uintptr(C.sizeof_struct_dbcapi_column_info*C.size_t(index))))

	if colInfo == nil {
		return errors.New(fmt.Sprintf("No column information was obtained for column %d", index+1))
	}

	colType := col._type
	values := new(bulkValue)
	values.dataType = colType
	values.nativeType = colInfo.native_type
	values.bulkValues = (*C.char)(unsafe.Pointer(col.buffer))
	values.nulls = (*C.dbcapi_bool)(unsafe.Pointer(col.is_null))
	values.lengths = (*C.size_t)(unsafe.Pointer(col.length))
	values.rowsetFetchSize = rows.rowsetFetchSize
	values.numberOfRows = uint(fetchedRows)
	values.precision = uint(col.buffer_size)

	switch colType {
	case C.A_BINARY:
		values.isBinary = (colInfo.native_type == C.DT_BINARY ||
			colInfo.native_type == C.DT_VARBINARY)
	case C.A_STRING:
		hanaType := colInfo.native_type
		if hanaType == C.DT_TIMESTAMP || hanaType == C.DT_LONGDATE ||
			hanaType == C.DT_SECONDDATE {
			values.isTimestamp = true
			values.isSecondDate = hanaType == C.DT_SECONDDATE
		} else if hanaType == C.DT_DATE || hanaType == C.DT_DAYDATE {
			values.isDate = true
		} else if hanaType == C.DT_TIME || hanaType == C.DT_SECONDTIME {
			values.isTime = true
		} else if colInfo.native_type == C.DT_DECIMAL ||
			colInfo.native_type == C.DT_FIXED16 {
			values.isDecimal = true
		} else { // Regular STRING column
			values.isString = true
		}
	case C.A_UVAL8:
		values.isBoolean = (colInfo.native_type == C.DT_BOOLEAN)
	case C.A_VAL8:
		fallthrough
	case C.A_VAL16:
		fallthrough
	case C.A_VAL32:
		fallthrough
	case C.A_UVAL16:
		fallthrough
	case C.A_UVAL32:
		fallthrough
	case C.A_VAL64:
		values.isInteger = (colInfo.native_type == C.DT_TINYINT ||
			colInfo.native_type == C.DT_SMALLINT || colInfo.native_type == C.DT_INT ||
			colInfo.native_type == C.DT_BIGINT)
	case C.A_DOUBLE:
		fallthrough
	case C.A_FLOAT:
		values.isFloat = (colInfo.native_type == C.DT_DOUBLE || colInfo.native_type == C.DT_REAL)
	default:
		return fmt.Errorf("Unknown column type: %d", colType)
	}

	*val = values
	return nil
}

func (rows *rows) getNextRowData(dest []driver.Value) error {
	col := (*C.struct_dbcapi_data_value)(C.malloc(C.sizeof_struct_dbcapi_data_value))
	defer C.free(unsafe.Pointer(col))

	for i := 0; i < len(dest); i++ {
		if rows.metadataReportsColumnIsLOB(uint(i)) && rows.getFetchUsingReadLOB() {
			rows.setupInternalDataForReadLOBHandling(&dest[i], i)
		} else {
			getCol := C.dbcapi_get_column(rows.dbcapi_stmt, C.dbcapi_u32(i), col)
			if getCol != 1 {
				return getError(rows.conn.dbcapi_conn)
			}

			err := getData(col, &dest[i], i, rows)

			if err != nil {
				return err
			}
		}
	}

	return nil
}

	func (rows *rows) metadataReportsColumnIsLOB(index uint) bool {
	if rows.boundColumnInfo == nil || uint(rows.boundColumnsCount) < index  {
		return false
	}

	columnInfo := (*C.struct_dbcapi_column_info)(unsafe.Pointer(uintptr(unsafe.Pointer(rows.boundColumnInfo)) +
		uintptr(C.sizeof_struct_dbcapi_column_info*C.size_t(index))))
	return (columnInfo.native_type == C.DT_CLOB) ||
		(columnInfo.native_type == C.DT_NCLOB) ||
		(columnInfo.native_type == C.DT_BLOB)
}

func (rows *rows) getFetchUsingReadLOB() bool {
	return rows.fetchUsingReadLob
}

func (row *rows) driverValueIsReadLOB(val *driver.Value) bool {
	v := reflect.ValueOf(val)
		_, ok := v.Interface().(*ReadLOB)
	return ok
}

func (rows *rows) setupInternalDataForReadLOBHandling(val *driver.Value, index int) {
	*val = &readLobValue{rows: rows, columnIndex: uint(index)}
}

func (rows *rows) returnBinaryValueDirectly(val *driver.Value,
	col *C.struct_dbcapi_data_value) {
	//Go 1.8 dostreamingInsertAndVerifyLobColumnses not have any lob types so we need to fetch the entire lob in memory
	bArr := make([]byte, int(*col.length))
	//Varbinary data in HANA can have a max size of 5000. If the size is greater than
	//5000, then we must be dealing with lobs.
	if *col.length <= MAX_BINARY_SIZE {
		copy(bArr, (*[MAX_BINARY_SIZE]byte)(unsafe.Pointer(col.buffer))[:])
		*val = bArr
	} else {
		*val = convertToLargeByteArray(col.buffer, uint(*col.length))
	}
}

func (rows *rows) returnStringValueDirectly(val *driver.Value,
	col *C.struct_dbcapi_data_value, index int) error {
	strPtr := (*C.char)(unsafe.Pointer(col.buffer))
	strVal := C.GoString(strPtr)
	colInfo := (*C.struct_dbcapi_column_info)(C.malloc(C.sizeof_struct_dbcapi_column_info))
	defer C.free(unsafe.Pointer(colInfo))
	colinfoRes := C.dbcapi_get_column_info(rows.dbcapi_stmt, C.dbcapi_u32(index), colInfo)

	if colinfoRes != 1 {
		return errors.New(fmt.Sprintf("Could not determine the column type for the: %d column", index+1))
	}

	hanaType := colInfo.native_type
	if hanaType == C.DT_TIMESTAMP || hanaType == C.DT_LONGDATE {
		tsVal, err := time.Parse(TS_LAYOUT_STRING, strVal)
		if err != nil {
			return err
		}
		*val = tsVal
	} else if hanaType == C.DT_SECONDDATE {
		sdVal, err := time.Parse(SECONDDATE_LAYOUT_STRING, strVal)
		if err != nil {
			return err
		}
		*val = sdVal
	} else if hanaType == C.DT_TIME || hanaType == C.DT_SECONDTIME {
		tVal, err := time.Parse(TIME_LAYOUT_STRING, strVal)
		if err != nil {
			return err
		}
		*val = tVal
	} else if hanaType == C.DT_DATE || hanaType == C.DT_DAYDATE {
		dVal, err := time.Parse(DATE_LAYOUT_STRING, strVal)
		if err != nil {
			return err
		}
		*val = dVal
	} else {
		*val = strVal
	}

	return nil
}

func getData(col *C.struct_dbcapi_data_value, val *driver.Value, index int, rows *rows) error {
	if col.is_null != nil && *col.is_null == 1 {
		*val = nil
		return nil
	}

	colType := col._type
	switch colType {
	case C.A_BINARY:
		rows.returnBinaryValueDirectly(val, col)
	case C.A_STRING:
		return rows.returnStringValueDirectly(val, col, index)
	case C.A_DOUBLE:
		doublePtr := (*float64)(unsafe.Pointer(col.buffer))
		*val = *doublePtr
	case C.A_VAL64:
		longPtr := (*int64)(unsafe.Pointer(col.buffer))
		*val = *longPtr
	case C.A_UVAL64:
		longPtr := (*uint64)(unsafe.Pointer(col.buffer))
		*val = *longPtr
	case C.A_VAL32:
		intPtr := (*int32)(unsafe.Pointer(col.buffer))
		*val = *intPtr
	case C.A_UVAL32:
		intPtr := (*uint32)(unsafe.Pointer(col.buffer))
		*val = *intPtr
	case C.A_VAL16:
		shortPtr := (*int16)(unsafe.Pointer(col.buffer))
		*val = *shortPtr
	case C.A_UVAL16:
		shortPtr := (*uint16)(unsafe.Pointer(col.buffer))
		*val = *shortPtr
	case C.A_VAL8:
		bytePtr := (*int8)(unsafe.Pointer(col.buffer))
		*val = *bytePtr
	case C.A_UVAL8:
		bytePtr := (*uint8)(unsafe.Pointer(col.buffer))
		*val = *bytePtr
	case C.A_FLOAT:
		floatPtr := (*float32)(unsafe.Pointer(col.buffer))
		*val = *floatPtr
	default:
		return errors.New("Unknown type")
	}

	return nil
}

func convertToLargeByteArray(buffer *C.char, size uint) []byte {
	res := make([]byte, size)
	s := res[:]
	ptr := uintptr(unsafe.Pointer(buffer))
	for size > MAX_BINARY_SIZE {
		copy(s, (*[MAX_BINARY_SIZE]byte)(unsafe.Pointer(ptr))[:])
		s = s[MAX_BINARY_SIZE:]
		size -= MAX_BINARY_SIZE
		ptr += MAX_BINARY_SIZE
	}
	if size > 0 {
		copy(s, (*[MAX_BINARY_SIZE]byte)(unsafe.Pointer(ptr))[:size])
	}
	return res
}

func (connection *connection) Ping(ctx context.Context) error {

	query := "SELECT 'PING' FROM SYS.DUMMY"
	sql := C.CString(query)
	defer C.free(unsafe.Pointer(sql))
	dbcapi_stmt := C.dbcapi_execute_direct(connection.dbcapi_conn, sql)

	detailedErrors := getPingDetailedErrorFromContext(ctx)

	if dbcapi_stmt != nil {
		defer C.dbcapi_free_stmt(dbcapi_stmt)
	}

	if detailedErrors {
		err := getError(connection.dbcapi_conn)

		if err != nil {
			// Could occur if a more detailed error occurs
			// e.g. Password policy change
			return err
		} else if dbcapi_stmt == nil {
			return driver.ErrBadConn
		}
	} else if dbcapi_stmt == nil {
		return driver.ErrBadConn
	}

	return nil
}

func (statement *statement) Close() error {
	C.dbcapi_free_stmt(statement.dbcapi_stmt)
	if statement.boundParameterInfos != nil {
		statement.boundParameterInfos.cleanup()
	}

	if statement.hasWriteLOBs() {
		statement.clearLOBs()
	}

	return nil
}

func (statement *statement) NumInput() int {
	// We don't let the sql package handle the validation, since we now support bulk parameters
	// passed in as an array of an array [][] interface{} so we return -1 here
	return -1
}

func (statement *statement) Exec(args []driver.Value) (driver.Result, error) {
	// This needs to find, from this level, whether or not we are dealing with [][]interface{}
	// or regular values.
	// 1) If we are dealing with regular parameters, proceed as normal
	// 2) If we are dealing with an [][]interface{} this will be the
	// only parameter [len(args) == 1].
	//    a) Bindings will be consumed up to how many rows contain the same types of columns
	//       consecutively. Once a row is found that is not similar, what has been batched so far is
	//       sent off
	//    b) Variable length arguments may use the metadata length reported to know the buffer size
	//       to use
	// 3) [][]interface{} and separate parameters cannot be mixed
	if statement.functionCode == 0 {
		statement.functionCode = int(C.dbcapi_get_function_code(statement.dbcapi_stmt))
	}
	argsLength := len(args)
	var err error
	if argsLength == 1 {
		v := reflect.ValueOf(args[0])
		if bulkValues, ok := v.Interface().([][]interface{}); ok {
			if (statement.functionCode != FunctionCode_INSERT) &&
				(statement.functionCode != FunctionCode_UPDATE) &&
				(statement.functionCode != FunctionCode_DELETE) {
				return nil, errors.New("Batching is only supported for Inserts/Updates/Deletes")
			}

			err = statement.executeBatchInternal(bulkValues)
		} else {
			err = statement.executeInternal(args)
		}
	} else {
		err = statement.executeInternal(args)
	}

	if err != nil {
		return nil, err
	}

	if statement.functionCode == FunctionCode_DDL {
		return driver.ResultNoRows, nil
	}
	rowsAffected := C.dbcapi_affected_rows(statement.dbcapi_stmt)
	if rowsAffected == -1 {
		return &result{rowsAffected: -1, err: getError(statement.conn.dbcapi_conn)}, nil
	} else {
		return &result{rowsAffected: int64(rowsAffected), err: nil}, nil
	}
}

func (statement *statement) ExecContext(ctx context.Context, args []driver.NamedValue) (driver.Result, error) {
	value := make([]driver.Value, len(args))
	for i := 0; i < len(args); i++ {
		value[i] = args[i].Value
	}

	deadline, ok := ctx.Deadline()
	if !ok {
		return statement.Exec(value)
	} else {
		err := setTimeout(statement, deadline.Sub(time.Now()).Seconds())
		if err != nil {
			return nil, err
		}
		return statement.Exec(value)
	}
}

type driverKey string

const (
	fetchSizeKey = driverKey("fetchsize")
	pingDetailedError = driverKey("pingDetailedError")
	fetchUsingReadLob = driverKey("fetchUsingReadLob")
)

func NewFetchSizeContext(ctx context.Context, fetchSize uint) (context.Context, error) {
	if fetchSize == 0 {
		return nil, errors.New("Provided fetch size must be >= 1")
	}

	return context.WithValue(ctx, fetchSizeKey, fetchSize), nil
}

func getFetchSizeFromContext(ctx context.Context) (uint, bool) {
	fetchSize, ok := ctx.Value(fetchSizeKey).(uint)
	return fetchSize, ok
}

func (statement *statement) setRowsetFetchSize(rowsetFetchSize uint) error {
	statement.rowsetFetchSize = rowsetFetchSize

	if rowsetFetchSize > 1 {
		ok := C.dbcapi_set_rowset_size(statement.dbcapi_stmt, C.dbcapi_u32(rowsetFetchSize))

		if ok != 1 {
			return getError(statement.conn.dbcapi_conn)
		}
	}

	return nil
}

func NewPingDetailedErrorContext(ctx context.Context) (context.Context) {
	return context.WithValue(ctx, pingDetailedError, true)
}

func getPingDetailedErrorFromContext(ctx context.Context) bool {
	pingDetailedError, ok := ctx.Value(pingDetailedError).(bool)

	if !ok {
		pingDetailedError = false
	}

	return pingDetailedError
}

func NewReadLobFetchContext(ctx context.Context) (context.Context) {
	return context.WithValue(ctx, fetchUsingReadLob, true)
}

func getFetchUsingReadLobFromContext(ctx context.Context) bool {
	fetchUsingReadLob, ok := ctx.Value(fetchUsingReadLob).(bool)

	if !ok {
		fetchUsingReadLob = false
	}

	return fetchUsingReadLob
}

func (statement *statement) setFetchUsingReadLob(fetchUsingReadLob bool) {
	statement.fetchUsingReadLob = fetchUsingReadLob
}

func (statement *statement) getFetchUsingReadLob() bool {
	return statement.fetchUsingReadLob
}

func (statement *statement) QueryContext(ctx context.Context, args []driver.NamedValue) (driver.Rows, error) {
	value := make([]driver.Value, len(args))
	for i := 0; i < len(args); i++ {
		value[i] = args[i].Value
	}

	deadline, ok := ctx.Deadline()

	if ok {
		err := setTimeout(statement, deadline.Sub(time.Now()).Seconds())
		if err != nil {
			return nil, err
		}
	}

	fetchSize, ok := getFetchSizeFromContext(ctx)

	if ok {
		if fetchSize == 0 {
			return nil, errors.New("Could not set fetchsize - the provided value was invalid")
		}

	} else {
		fetchSize = 0 // Represents usage of non-batch types
	}
	statement.setRowsetFetchSize(fetchSize)


	fetchUsingReadLob := getFetchUsingReadLobFromContext(ctx)
	statement.setFetchUsingReadLob(fetchUsingReadLob)

	return statement.Query(value)
}

func setTimeout(statement *statement, timeout float64) error {
	//Add +1 since float64 to int32 will cause some loss in precision
	timeoutRes := C.dbcapi_set_query_timeout(statement.dbcapi_stmt, C.dbcapi_i32(timeout+1))
	if timeoutRes != 1 {
		return getError(statement.conn.dbcapi_conn)
	}

	return nil
}

func (statement *statement) Query(args []driver.Value) (driver.Rows, error) {
	numberOfParameters := len(args)
	expectedNumberOfParams := int(C.dbcapi_num_params(statement.dbcapi_stmt))

	if numberOfParameters != expectedNumberOfParams {
		return nil, fmt.Errorf("Expected number of input parameters: %d, got: %d",
			expectedNumberOfParams, numberOfParameters)
	}

	for i := 0; i < len(args); i++ {
		bpv, err := statement.bindParameter(i, args[i])
		if bpv != nil {
			defer bpv.cleanup()
		}

		if err != nil {
			return nil, err
		}
	}

	execRes := C.dbcapi_execute(statement.dbcapi_stmt)
	if execRes != 1 {
		return nil, getError(statement.conn.dbcapi_conn)
	}

	return statement.newResultSetRows()
}

func (statement *statement) newResultSetRows() (*rows, error) {
	resultSetRows := &rows{dbcapi_stmt: statement.dbcapi_stmt, conn: statement.conn,
							isFromStatement: true, isClosed: false, err: nil,
							rowsetFetchSize: statement.rowsetFetchSize,
							fetchUsingReadLob: statement.getFetchUsingReadLob()}

	isBulkFetch := statement.rowsetFetchSize >= 1

	err := resultSetRows.doSetupForFetching(isBulkFetch)

	if err != nil {
		resultSetRows.Close()
		return nil, err
	}

	return resultSetRows, nil
}

func (result *result) LastInsertId() (int64, error) {
	return -1, errors.New("Feature not supported")
}

func (result *result) RowsAffected() (int64, error) {
	if result.err != nil {
		return 0, result.err
	}
	return result.rowsAffected, nil
}

func (statement *statement) executeInternal(args []driver.Value) error {
	numberOfParameters := len(args)
	expectedNumberOfParams := int(C.dbcapi_num_params(statement.dbcapi_stmt))

	if numberOfParameters != expectedNumberOfParams {
		return fmt.Errorf("Expected number of input parameters: %d, got: %d",
			expectedNumberOfParams, numberOfParameters)
	}

	statement.clearLOBs()

	for i := 0; i < len(args); i++ {
		bpv, err := statement.bindParameter(i, args[i])
		defer bpv.cleanup()

		if err != nil {
			return err
		}
	}

	execRes := C.dbcapi_execute(statement.dbcapi_stmt)


	var err error

	if execRes != 1 {
		err = getError(statement.conn.dbcapi_conn)
	}

	if statement.hasWriteLOBs() {
		if err != nil {
			statement.clearLOBs()
		} else {
			err = statement.sendDataFromWriteLOBs()
		}
	}

	return err
}

func (statement *statement) hasWriteLOBs() bool {
	return (statement.functionCode == FunctionCode_DELETE ||
		statement.functionCode == FunctionCode_INSERT ||
		statement.functionCode == FunctionCode_UPDATE) && (len(statement.storedWriteLobs) > 0)
}

func (statement *statement) storeLOBForColumnOfSingleRow(columnIndex uint, lob *WriteLOB) {
	statement.storeLOBForColumn(0, columnIndex, lob)
}

func (statement *statement) storeLOBForColumn(rowIndex uint, columnIndex uint, lob *WriteLOB) {
	rowWriteLOBColumnsList, ok := statement.storedWriteLobs[rowIndex]

	if !ok {
		rowWriteLOBColumnsList = list.New()
		statement.storedWriteLobs[rowIndex] = rowWriteLOBColumnsList
	}

	lob.setParameterIndex(columnIndex)
	rowWriteLOBColumnsList.PushBack(lob)
}

func (statement *statement) clearLOBs() {
	// We don't care about the sequencing of rows here - this close() is just
	// resetting the actual WriteLOB object internally, it is not sending any
	// protocol information
	for rowIndex, rowWriteLOBColumnsList := range statement.storedWriteLobs {
		for el := rowWriteLOBColumnsList.Front(); el != nil; el = el.Next() {
			val := reflect.ValueOf(el.Value)

			if val.IsValid() {
				writeLOB, ok := val.Interface().(*WriteLOB)

				if ok {
					writeLOB.close()
				}
			}
		}

		statement.storedWriteLobs[rowIndex] = nil
	}
}

// NOTE: We are doing regular loop by index from 0 here since we know that
// map indices are stored as sequential row indices starting from 0,
// but we don't know if iteration of the map via regular means
// will be in order of the row indices
// Lobs are therefore guaranteed to be processed in increasing order of
// increasing row indices, and for a particular row index, in increasing
// order of column indices.
// This is also being done sequentially. If in future, the server allows for
// concurrent sending of data, this can be modified to process multiple
// WriteLOBs in parallel
func (statement *statement) sendDataFromWriteLOBs() error {
	var err error

	for rowIndex := 0; rowIndex <  len(statement.storedWriteLobs); rowIndex++ {
		rowWriteLOBColumnsList := statement.storedWriteLobs[uint(rowIndex)]

		for el := rowWriteLOBColumnsList.Front(); el != nil; el = el.Next() {
			val := reflect.ValueOf(el.Value)

			if val.IsValid() {
				lobValue, ok := val.Interface().(*WriteLOB)

				if ok {
					if err != nil {
						// Close the other Lobs before returning the error,
						// otherwise the other writers from the producer will continue
						// to block that thread forever!
						lobValue.close()
					} else {
						err = statement.processWriteLOB(lobValue)
					}
				}
			}
		}
	}

	// Set the storedWriteLobs to nil in case future usages of the statement
	// use different LOBs
	statement.storedWriteLobs = nil
	return err
}

func (statement *statement) processWriteLOB(writeLOB *WriteLOB) error {
	defer writeLOB.close()
	return writeLOB.sendInputParameterData(statement)
}

//This function can be optimized since dbcapi is fixed now. The caller can allocate
//a buffer once and pass it to the function, rather then this function allocating
//buffer for a single use.
func (statement *statement) bindParameter(index int, paramVal driver.Value) (*boundParameterValue, error) {
	// All pointer memory is to be deallocated by the caller
	bpv := new(boundParameterValue)
	bpv.boundStruct = (*C.struct_dbcapi_bind_data)(C.calloc(C.size_t(1), C.sizeof_struct_dbcapi_bind_data))
	descRes := C.dbcapi_describe_bind_param(statement.dbcapi_stmt, C.dbcapi_u32(index), bpv.boundStruct)

	if descRes != 1 {
		return bpv, getError(statement.conn.dbcapi_conn)
	}

	bpv.boundStruct.value.is_address = C.dbcapi_bool(0)
	if paramVal == nil {
		nullIndicator := (*C.dbcapi_bool)(C.malloc(C.sizeof_dbcapi_bool))
		*nullIndicator = C.dbcapi_bool(1)
		bpv.boundStruct.value.is_null = nullIndicator
	} else {
		v := reflect.ValueOf(paramVal)
		length := (*C.size_t)(C.malloc(C.sizeof_size_t))
		bpv.boundStruct.value.length = length

		switch v.Kind() {
		case reflect.Uint8:
			fallthrough
		case reflect.Uint16:
			fallthrough
		case reflect.Uint32:
			fallthrough
		case reflect.Uint64:
			fallthrough
		case reflect.Uint:
			*length = C.size_t(8)
			iVal := (*C.uint64_t)(C.malloc(C.sizeof_uint64_t))
			*iVal = C.uint64_t(v.Uint())
			bpv.boundStruct.value.buffer = (*C.char)(unsafe.Pointer(iVal))
			bpv.boundStruct.value._type = C.A_UVAL64
		case reflect.Int8:
			fallthrough
		case reflect.Int16:
			fallthrough
		case reflect.Int32:
			fallthrough
		case reflect.Int64:
			fallthrough
		case reflect.Int:
			*length = C.size_t(8)
			iVal := (*C.int64_t)(C.malloc(C.sizeof_int64_t))
			*iVal = C.int64_t(v.Int())
			bpv.boundStruct.value.buffer = (*C.char)(unsafe.Pointer(iVal))
			bpv.boundStruct.value._type = C.A_VAL64
		case reflect.Bool:
			*length = C.size_t(1)
			bVal := (*C.bool)(C.malloc(C.sizeof_bool))
			*bVal = C.bool(v.Bool())
			bpv.boundStruct.value.buffer = (*C.char)(unsafe.Pointer(bVal))
			bpv.boundStruct.value._type = C.A_VAL8
		case reflect.Float32:
			fallthrough
		case reflect.Float64:
			*length = C.size_t(8)
			dVal := (*C.double)(C.malloc(C.sizeof_double))
			*dVal = C.double(v.Float())
			bpv.boundStruct.value.buffer = (*C.char)(unsafe.Pointer(dVal))
			bpv.boundStruct.value._type = C.A_DOUBLE
		case reflect.String:
			strVal := v.String()
			stringLen := len(strVal)
			a := make([]byte, stringLen+1)
			copy(a[:], strVal)
			bpv.boundStruct.value.buffer = (*C.char)(C.malloc(C.size_t(stringLen + 1)))
			C.memset(unsafe.Pointer(bpv.boundStruct.value.buffer), C.int(0), C.size_t(stringLen+1))
			C.strncpy(bpv.boundStruct.value.buffer, (*C.char)(unsafe.Pointer(&a[0])), C.size_t(stringLen))
			*length = C.size_t(stringLen)
			bpv.boundStruct.value._type = C.A_STRING
		case reflect.Slice:
			if bArr, ok := v.Interface().([]byte); ok {
				bArrLen := len(bArr)
				*length = C.size_t(bArrLen)
				if len(bArr) > 0 {
					bpv.boundStruct.value.buffer = (*C.char)(C.malloc(C.size_t(bArrLen)))
					C.memcpy(unsafe.Pointer(bpv.boundStruct.value.buffer), unsafe.Pointer(&bArr[0]), C.size_t(bArrLen))
				} else {
					a := make([]byte, 1)
					bpv.boundStruct.value.buffer = (*C.char)(C.malloc(C.size_t(1)))
					C.memcpy(unsafe.Pointer(bpv.boundStruct.value.buffer), unsafe.Pointer(&a[0]), C.size_t(1))
				}
				bpv.boundStruct.value._type = C.A_BINARY
			} else {
				return nil, errors.New("Unknown type")
			}
		default:
			if tVal, ok := v.Interface().(time.Time); ok {
				err := statement.createStringBufferForTimeValue(bpv, tVal, index)

				if err != nil {
					return bpv, err
				}
			} else if writeLOB, ok := v.Interface().(*WriteLOB); ok {
				// We hold all Write LOBs for after the call to dbcapi_execute()
				// to subsequently and sequentially send their data via
				// dbcapi_send_param_data().
				// See sendretrievepartblob.SendRetrievePartBlob.run()
				err := statement.bindBufferForDataAfterExecute(bpv, writeLOB,
					index, length)

				if err != nil {
					return bpv, err
				}

				statement.storeLOBForColumnOfSingleRow(uint(index), writeLOB)
			} else {
				return bpv, fmt.Errorf("Unknown type: %s ", v.Kind())
			}
		}
	}

	bindRes := C.dbcapi_bind_param(statement.dbcapi_stmt, C.dbcapi_u32(index), bpv.boundStruct)
	if bindRes != 1 {
		return bpv, getError(statement.conn.dbcapi_conn)
	}
	return bpv, nil
}

func (statement *statement) createStringBufferForTimeValue(bpv *boundParameterValue,
	tVal time.Time, index int) error {
	paramInfo := (*C.dbcapi_bind_param_info)(C.malloc(C.sizeof_struct_dbcapi_bind_param_info))
	defer C.free(unsafe.Pointer(paramInfo))
	paramInfoResult := C.dbcapi_get_bind_param_info(statement.dbcapi_stmt, C.dbcapi_u32(index), paramInfo)

	if paramInfoResult == C.dbcapi_bool(0) {
		return errors.New("Could not get Parameter Metadata for the column")
	}

	var insertTimeVal time.Time

	if statement.conn.dsnInfo.ConvertTimestampToUTC() {
		insertTimeVal = tVal.UTC()
	} else {
		insertTimeVal = tVal
	}

	strVal := ""

	if (insertTimeVal.Year() == 0) && //Server does not support the year = 0 for DATE and TIMESTAMP type columns
		((paramInfo.native_type == C.dbcapi_native_type(C.DT_DAYDATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_DATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_LONGDATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDDATE))) {
		if statement.conn.dsnInfo.ConvertTimestampToUTC() {
			return fmt.Errorf(
				"Specified date '%v' would result in a UTC date '%v' with year = 0, which is not supported by the server",
				tVal, insertTimeVal)
		}
		return fmt.Errorf("Specified date '%v' with year = 0 is not supported by the server", insertTimeVal)
	}
	if paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDTIME) || // TIME
		paramInfo.native_type == C.dbcapi_native_type(C.DT_TIME) {
		strVal = insertTimeVal.Format(TIME_LAYOUT_STRING)
	} else if paramInfo.native_type == C.dbcapi_native_type(C.DT_DAYDATE) || // DATE
		paramInfo.native_type == C.dbcapi_native_type(C.DT_DATE) {
		strVal = insertTimeVal.Format(DATE_LAYOUT_STRING)
	} else if paramInfo.native_type == C.dbcapi_native_type(C.DT_LONGDATE) || // TIMESTAMP
		paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDDATE) {
		if insertTimeVal.Nanosecond() == 0 {
			strVal = insertTimeVal.Format(SECONDDATE_LAYOUT_STRING)
		} else {
			strVal = insertTimeVal.Format(TS_LAYOUT_STRING)
		}
	} else { // Non-DATE|TIME|TIMESTAMP type - let the conversion success/failure be determined by lower levels
		strVal = insertTimeVal.Format(TS_LAYOUT_STRING)
	}

	stringLen := len(strVal)
	a := make([]byte, stringLen+1)
	copy(a[:], strVal)
	bpv.boundStruct.value.buffer = (*C.char)(C.malloc(C.size_t(stringLen + 1)))
	C.memset(unsafe.Pointer(bpv.boundStruct.value.buffer), C.int(0), C.size_t(stringLen+1))
	C.strncpy(bpv.boundStruct.value.buffer, (*C.char)(unsafe.Pointer(&a[0])), C.size_t(stringLen))
	*bpv.boundStruct.value.length = C.size_t(stringLen)
	bpv.boundStruct.value._type = C.A_STRING
	return nil
}

func (statement *statement) bindBufferForDataAfterExecute(bpv *boundParameterValue,
	writeLOB *WriteLOB, index int, length *C.size_t) error {
	paramInfo :=
		(*C.dbcapi_bind_param_info)(C.malloc(C.sizeof_struct_dbcapi_bind_param_info))
	defer C.free(unsafe.Pointer(paramInfo))
	paramInfoResult := C.dbcapi_get_bind_param_info(statement.dbcapi_stmt,
		C.dbcapi_u32(index), paramInfo)

	if paramInfoResult == C.dbcapi_bool(0) {
		return fmt.Errorf("Could not get Parameter Metadata for the column %d",
			index)
	}

	// If multibyte, UTF-8 data is provided for a CLOB column,
	// its handling will be dependent on dbcapi/SQLDBC's handling.
	// No validation/conversion is done here
	if((paramInfo.native_type == C.dbcapi_native_type(C.DT_CLOB)) ||
		(paramInfo.native_type == C.dbcapi_native_type(C.DT_NCLOB))) {
		bpv.boundStruct.value._type = C.A_STRING
	} else if(paramInfo.native_type == C.dbcapi_native_type(C.DT_BLOB)) {
		bpv.boundStruct.value._type = C.A_BINARY
	} else { // Error - not a lob column
		return fmt.Errorf("LOB reader is being provided for a non-LOB column type: %d",
			int(paramInfo.native_type))
	}

	// Signal this is Data provided after the execute call
	// This is implicitly done via the calloc() call initialization, earlier
	// but it is done here to ensure everything is correct
	*length = C.size_t(2147483647)
	bpv.boundStruct.value.length = length
	bpv.boundStruct.value.buffer = nil
	return nil
}

func (statement *statement) executeBatchInternal(bulkValues [][]interface{}) error {
	if statement.err != nil {
		return statement.err
	}
	// Bindings will be consumed up to how many rows contain the same types of columns
	// consecutively. Once a row is found that is not similar, what has been batched so far is
	// sent off and the logic continues with the next set of rows until the batch is consumed.
	// It is recommended to use the same types in all columns for all rows, for maximum efficiency.
	rows := len(bulkValues)

	if rows == 0 {
		return errors.New("No values provided")
	}

	var (
		err error
		// boundParameterInfos *paramInfoValues
	)

	if statement.boundParameterInfos == nil {
		statement.err = statement.retrieveParamInfos()

		if statement.err != nil {
			return statement.err
		}
	}

	for rowIndex := 0; rowIndex < rows; {
		startRow := rowIndex
		var startingRowValues []reflect.Value
		startingRowValues, err =
			statement.getColumnTypesForRow(bulkValues[startRow], rowIndex)

		if err != nil {
			return err
		}
		rowIndex++

		for rowIndex < rows {
			include, err := statement.rowCanBeIncludedInBatch(bulkValues[rowIndex],
				startingRowValues, rowIndex)

			if err != nil {
				return err
			} else if !include {
				break
			}

			rowIndex++
		}

		if err != nil {
			return err
		}

		res := C.dbcapi_set_batch_size(statement.dbcapi_stmt, C.dbcapi_u32(rowIndex-startRow))

		if res != 1 {
			return getError(statement.conn.dbcapi_conn)
		}

		err = statement.bindBulkParametersAndExecute(startRow, rowIndex-1, startingRowValues,
			bulkValues)

		if err != nil {
			return err
		}
	}

	return nil
}

func (statement *statement) getColumnTypesForRow(row []interface{}, rowIndex int) ([]reflect.Value, error) {
	rowLength := len(row)
	expectedNumberOfParams := len(statement.boundParameterInfos.paramInfos)

	if rowLength != expectedNumberOfParams {
		return nil, fmt.Errorf("Expected number of input parameters: %d, got: %d",
			expectedNumberOfParams, rowLength)
	}

	columnValues := make([]reflect.Value, rowLength)

	var err error
	for columnIndex, column := range row {
		columnValue := reflect.ValueOf(column)

		if columnValue.IsValid() {
			err = statement.isAcceptableInputParameterType(columnValue, rowIndex, columnIndex)

			if err != nil {
				break
			}
		}
		columnValues[columnIndex] = columnValue
	}

	return columnValues, err
}

func (statement *statement) retrieveParamInfos() error {
	if statement.boundParameterInfos != nil {
		return nil
	}
	boundParameterInfos := new(paramInfoValues)
	// Deallocated by boundParameterInfos.cleanup()
	expectedNumberOfParams := int(C.dbcapi_num_params(statement.dbcapi_stmt))

	if expectedNumberOfParams == 0 {
		return errors.New("This statement execution does not expect any input/output parameters")
	}

	boundParameterInfos.paramInfos = make([]*C.dbcapi_bind_param_info, expectedNumberOfParams)

	var err error

	for columnIndex := 0; columnIndex < expectedNumberOfParams; columnIndex++ {
		boundParameterInfos.paramInfos[columnIndex] = (*C.dbcapi_bind_param_info)(C.calloc(C.size_t(1),
			C.sizeof_struct_dbcapi_bind_param_info))
		paramInfoResult := C.dbcapi_get_bind_param_info(statement.dbcapi_stmt,
			C.dbcapi_u32(columnIndex), boundParameterInfos.paramInfos[columnIndex])

		if paramInfoResult == C.dbcapi_bool(0) {
			err = fmt.Errorf("Could not get Parameter Metadata for the column %d: %s",
				columnIndex+1, getError(statement.conn.dbcapi_conn))
			break
		}
	}

	statement.boundParameterInfos = boundParameterInfos
	return err
}

func (statement *statement) retrieveParamInfo(columnIndex int) *C.dbcapi_bind_param_info {
	return statement.boundParameterInfos.paramInfos[columnIndex]
}

func (statement *statement) rowCanBeIncludedInBatch(row []interface{},
	startingRowValues []reflect.Value, rowIndex int) (bool, error) {
	if len(row) != len(startingRowValues) {
		return false, errors.New("All of the batch rows do not have the same length")
	}

	canBeBatched := true

	for columnIndex, columnInterface := range row {
		columnValue := reflect.ValueOf(columnInterface)
		startingRowValue := startingRowValues[columnIndex]

		// nil (NULL) matches any previous column value type
		if !startingRowValue.IsValid() {
			// If any of the startingRowValues are nil and the current row's column value is not
			// nil, then set the startingRowValue to the value of the current row's column - it will
			// be used in future comparisons
			if columnValue.IsValid() {
				startingRowValues[columnIndex] = columnValue
			}
		} else if columnValue.IsValid() {
			err := statement.isAcceptableInputParameterType(columnValue, rowIndex, columnIndex)

			if err != nil {
				return false, fmt.Errorf("Unsupported type (row %d, column %d)", rowIndex+1,
					columnIndex+1)
			}

			canBeBatched = columnValue.Kind() == startingRowValue.Kind()
		} // !columnValue.isValid() => nil [NULL] (which matches any type)

		if !canBeBatched {
			break
		}
	}
	return canBeBatched, nil
}

func (statement *statement) isAcceptableInputParameterType(value reflect.Value,
	rowIndex int, columnIndex int) error {
	paramInfo := statement.retrieveParamInfo(columnIndex)

	if paramInfo == nil {
		return fmt.Errorf("Unable to get the Param Info for column %d", columnIndex)
	}

	switch value.Kind() {
	case reflect.Slice:
		if _, ok := value.Interface().([]byte); !ok {
			return fmt.Errorf("Unsupported type (row %d, column %d)", rowIndex+1, columnIndex+1)
		}
		fallthrough
	case reflect.Uint8:
		fallthrough
	case reflect.Uint16:
		fallthrough
	case reflect.Uint32:
		fallthrough
	case reflect.Uint64:
		fallthrough
	case reflect.Uint:
		fallthrough
	case reflect.Int8:
		fallthrough
	case reflect.Int16:
		fallthrough
	case reflect.Int32:
		fallthrough
	case reflect.Int64:
		fallthrough
	case reflect.Int:
		fallthrough
	case reflect.Bool:
		fallthrough
	case reflect.Float32:
		fallthrough
	case reflect.Float64:
		fallthrough
	case reflect.String:
		return nil
	default:
		if tVal, ok := value.Interface().(time.Time); ok {
			var insertTimeVal time.Time

			if statement.conn.dsnInfo.ConvertTimestampToUTC() {
				insertTimeVal = tVal.UTC()
			} else {
				insertTimeVal = tVal
			}

			// Server does not support the year = 0 for DATE and TIMESTAMP type columns
			if (insertTimeVal.Year() == 0) &&
				((paramInfo.native_type == C.dbcapi_native_type(C.DT_DAYDATE)) ||
					(paramInfo.native_type == C.dbcapi_native_type(C.DT_DATE)) ||
					(paramInfo.native_type == C.dbcapi_native_type(C.DT_LONGDATE)) ||
					(paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDDATE))) {
				if statement.conn.dsnInfo.ConvertTimestampToUTC() {
					return fmt.Errorf(
						"Specified date '%v' would result in a UTC date '%v' with year = 0, "+
							"which is not supported by the server", tVal, insertTimeVal)
				}
				return fmt.Errorf("Specified date '%v' with year = 0 is not supported "+
					"by the server", insertTimeVal)
			}
			return nil
		}
		return fmt.Errorf("Unsupported type (row %d, column %d)", rowIndex+1, columnIndex+1)
	}
}

func (statement *statement) bindBulkParametersAndExecute(startRow int, endRow int,
	columnValues []reflect.Value, bulkValues [][]interface{}) error {

	boundColumns, err := statement.bindBulkParameters(startRow, endRow, columnValues, bulkValues)

	for _, boundColumn := range boundColumns {
		if boundColumn != nil {
			defer boundColumn.cleanup()
		}
	}

	if err != nil {
		return err
	}

	execRes := C.dbcapi_execute(statement.dbcapi_stmt)

	if execRes != 1 {
		return getError(statement.conn.dbcapi_conn)
	}

	return nil
}

func (statement *statement) bindBulkParameters(startRow int, endRow int,
	columnValues []reflect.Value, bulkValues [][]interface{}) ([]*boundParameterValue, error) {
	numberOfColumns := len(columnValues)
	// boundColumns always needs to be returned as the memory is deallocated by the caller using
	// using defer boundColumns[index].cleanup()
	boundColumns := make([]*boundParameterValue, numberOfColumns)
	var err error

	// Loop through the columns kinds, and make a buffer array for it
	for i := 0; i < numberOfColumns; i++ {
		columnValue := columnValues[i]

		var bpv *boundParameterValue
		var err error
		if !columnValue.IsValid() {
			// this is only the case if all of the rows for a particular column had nil values
			bpv, err = statement.bindBulkNullValuesForColumn(i, startRow, endRow)
		} else {
			bpv, err = statement.bindBulkRegularParameters(i, columnValue, bulkValues, startRow,
				endRow)
		}

		boundColumns[i] = bpv

		if err != nil {
			break
		}
	}
	return boundColumns, err
}

func (statement *statement) bindBulkNullValuesForColumn(columnIndex int, startRow int,
	endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		*length = 0
		nullIndicator :=
			(*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
				uintptr(nullSize*C.size_t(outputValueIndex))))
		*nullIndicator = C.dbcapi_bool(1)
	}

	bindRes := C.dbcapi_bind_param(statement.dbcapi_stmt, C.dbcapi_u32(columnIndex), bpv.boundStruct)

	if bindRes != 1 {
		return bpv, getError(statement.conn.dbcapi_conn)
	}

	return bpv, nil
}

func (statement *statement) bindBulkRegularParameters(columnIndex int,
	columnValue reflect.Value, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	var bpv *boundParameterValue
	var err error

	switch columnValue.Kind() {
	case reflect.Uint8:
		fallthrough
	case reflect.Uint16:
		fallthrough
	case reflect.Uint32:
		fallthrough
	case reflect.Uint64:
		fallthrough
	case reflect.Uint:
		bpv, err = statement.bindIntegerBatch(columnIndex, bulkValues, startRow, endRow, false)
	case reflect.Int8:
		fallthrough
	case reflect.Int16:
		fallthrough
	case reflect.Int32:
		fallthrough
	case reflect.Int64:
		fallthrough
	case reflect.Int:
		bpv, err = statement.bindIntegerBatch(columnIndex, bulkValues, startRow, endRow, true)
	case reflect.Float32:
		fallthrough
	case reflect.Float64:
		bpv, err = statement.bindFloatBatch(columnIndex, bulkValues, startRow, endRow)
	case reflect.Bool:
		bpv, err = statement.bindBooleanBatch(columnIndex, bulkValues, startRow, endRow)
	case reflect.String:
		bpv, err = statement.bindStringBatch(columnIndex, bulkValues, startRow, endRow)
	case reflect.Slice:
		bpv, err = statement.bindByteArrayBatch(columnIndex, bulkValues, startRow, endRow)
	default: {
		if _, ok := columnValue.Interface().(time.Time); ok {
			// Time batch (validated in rowCanBeIncludedInBatch())
			bpv, err = statement.bindTimeBatch(columnIndex, bulkValues, startRow, endRow)
		}
	}
	}

	if err != nil {
		return bpv, err
	}

	bindRes := C.dbcapi_bind_param(statement.dbcapi_stmt, C.dbcapi_u32(columnIndex), bpv.boundStruct)

	if bindRes != 1 {
		return bpv, getError(statement.conn.dbcapi_conn)
	}
	return bpv, nil
}

func (statement *statement) initializeBatchBoundParameterValuesForColumn(columnIndex int,
	batchSize int) (*boundParameterValue, error) {
	return statement.initializeBatchBoundParameterValuesForColumnInternal(columnIndex,
		batchSize, false)
}

func (statement *statement) initializeBatchBoundParameterValuesForColumnInternal(columnIndex int,
	batchSize int, forDataAtExecute bool) (*boundParameterValue, error) {
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv := new(boundParameterValue)
	bpv.isBatch = true
	bpv.batchSize = uint64(batchSize)
	bpv.boundStruct = (*C.struct_dbcapi_bind_data)(C.calloc(C.size_t(1), C.sizeof_struct_dbcapi_bind_data))
	descRes := C.dbcapi_describe_bind_param(statement.dbcapi_stmt, C.dbcapi_u32(columnIndex),
		bpv.boundStruct)

	if descRes != 1 {
		return bpv, getError(statement.conn.dbcapi_conn)
	}

	bpv.boundStruct.value.is_address = C.dbcapi_bool(1)
	if forDataAtExecute {
		bpv.boundStruct.value.buffer = nil
	} else {
		bpv.boundStruct.value.buffer = (*C.char)(C.calloc(C.size_t(1), pointerSize*C.size_t(batchSize)))
	}
	nullIndicators := (*C.dbcapi_bool)(C.calloc(C.size_t(1), nullSize*C.size_t(batchSize)))
	bpv.boundStruct.value.is_null = nullIndicators
	lengths := (*C.size_t)(C.calloc(C.size_t(1), C.size_t(batchSize)*C.sizeof_size_t))
	bpv.boundStruct.value.length = lengths
	return bpv, nil
}

func (statement *statement) bindIntegerBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int, signed bool) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length

	valueLen := C.size_t(C.sizeof_int64_t)
	iVals := bpv.boundStruct.value.buffer

	if signed {
		bpv.boundStruct.value._type = C.A_VAL64
	} else {
		bpv.boundStruct.value._type = C.A_UVAL64
	}

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		*length = valueLen
		nullIndicator :=
			(*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
				uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			*nullIndicator = C.dbcapi_bool(0)
			if signed {
				iVal := (*C.int64_t)(C.malloc(C.size_t(C.sizeof_int64_t)))
				*iVal = C.int64_t(theValue.Int())
				iValPos := (**C.int64_t)(unsafe.Pointer(uintptr(unsafe.Pointer(iVals)) +
					uintptr(pointerSize*C.size_t(outputValueIndex))))
				*iValPos = iVal
			} else {
				iVal := (*C.uint64_t)(C.malloc(C.size_t(C.sizeof_uint64_t)))
				*iVal = C.uint64_t(theValue.Uint())
				iValPos := (**C.uint64_t)(unsafe.Pointer(uintptr(unsafe.Pointer(iVals)) +
					uintptr(pointerSize*C.size_t(outputValueIndex))))
				*iValPos = iVal
			}
		} else {
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, nil
}

func (statement *statement) bindFloatBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length

	valueLen := C.size_t(C.sizeof_int64_t)
	dVals := (**C.double)(unsafe.Pointer(bpv.boundStruct.value.buffer))
	bpv.boundStruct.value._type = C.A_DOUBLE

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		*length = valueLen
		nullIndicator :=
			(*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
				uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			*nullIndicator = C.dbcapi_bool(0)
			dVal := (*C.double)(C.malloc(C.size_t(C.sizeof_double)))
			*dVal = C.double(theValue.Float())
			dValPos := (**C.double)(unsafe.Pointer(uintptr(unsafe.Pointer(dVals)) +
				uintptr(pointerSize*C.size_t(outputValueIndex))))
			*dValPos = dVal
		} else {
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, nil
}

func (statement *statement) bindBooleanBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length

	valueLen := C.size_t(C.sizeof_uint8_t)
	bVals := (**C.uint8_t)(unsafe.Pointer(bpv.boundStruct.value.buffer))
	bpv.boundStruct.value._type = C.A_VAL8

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		*length = valueLen
		nullIndicator :=
			(*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
				uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			*nullIndicator = C.dbcapi_bool(0)
			bVal := (*C.uint8_t)(C.malloc(C.size_t(C.sizeof_uint8_t)))
			if theValue.Bool() {
				*bVal = C.uint8_t(1)
			} else {
				*bVal = C.uint8_t(0)
			}
			bValPos := (**C.uint8_t)(unsafe.Pointer(uintptr(unsafe.Pointer(bVals)) +
				uintptr(pointerSize*C.size_t(outputValueIndex))))
			*bValPos = bVal
		} else {
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, nil
}

func (statement *statement) bindStringBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length

	strVals := (**C.char)(unsafe.Pointer(bpv.boundStruct.value.buffer))
	bpv.boundStruct.value._type = C.A_STRING

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		nullIndicator :=
			(*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
				uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			*length = C.size_t(len(theValue.String()))
			*nullIndicator = C.dbcapi_bool(0)
			strValPos := (**C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(strVals)) +
				uintptr(pointerSize*C.size_t(outputValueIndex))))
			*strValPos = C.CString(theValue.String())
		} else {
			*length = 0
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, nil
}

func (statement *statement) bindByteArrayBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length
	bArrVals := (**C.char)(unsafe.Pointer(bpv.boundStruct.value.buffer))
	bpv.boundStruct.value._type = C.A_BINARY

	for i := startRow; i <= endRow; i++ {
		outputValueIndex := i - startRow
		// The check for this is a []byte done earlier [in isAcceptableInputParameterType()],
		// here we just use the value here
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		nullIndicator := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
			uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			bArrLen := len(theValue.Bytes())
			*length = C.size_t(bArrLen)
			bArrValPos := (**C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(bArrVals)) +
				uintptr(pointerSize*C.size_t(outputValueIndex))))
			var bArrVal *C.char

			if bArrLen > 0 {
				bArrVal = (*C.char)(C.malloc(C.size_t(bArrLen)))
				C.memcpy(unsafe.Pointer(bArrVal), unsafe.Pointer(&theValue.Bytes()[0]),
					C.size_t(bArrLen))
			} else {
				a := make([]byte, 1)
				bArrVal = (*C.char)(C.malloc(C.size_t(1)))
				C.memcpy(unsafe.Pointer(bArrVal), unsafe.Pointer(&a[0]), C.size_t(1))
			}
			*bArrValPos = bArrVal
		} else {
			*length = 0
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, nil
}

func (statement *statement) bindTimeBatch(columnIndex int, bulkValues [][]interface{},
	startRow int, endRow int) (*boundParameterValue, error) {
	batchSize := endRow - startRow + 1
	// bpv always needs to be returned as the memory is deallocated by the caller using
	// using defer bpv.cleanup()
	bpv, err := statement.initializeBatchBoundParameterValuesForColumn(columnIndex, batchSize)

	if err != nil {
		return bpv, err
	}

	nullIndicators := bpv.boundStruct.value.is_null
	lengths := bpv.boundStruct.value.length
	strVals := (**C.char)(unsafe.Pointer(bpv.boundStruct.value.buffer))
	bpv.boundStruct.value._type = C.A_STRING
	var firstError error

	for i := startRow; i <= endRow; i++ {
		theValue := reflect.ValueOf(bulkValues[i][columnIndex])
		outputValueIndex := i - startRow
		length :=
			(*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(lengths)) +
				uintptr(C.sizeof_size_t*C.size_t(outputValueIndex))))
		nullIndicator := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(nullIndicators)) +
			uintptr(nullSize*C.size_t(outputValueIndex))))
		if theValue.IsValid() {
			// This validation is always done in rowCanBeIncludedInBatch(), so we can just get the
			// value here
			tVal, _ := theValue.Interface().(time.Time)
			strVal, err := statement.getAdjustedStringValueFromTime(i, columnIndex, tVal)

			strValPos := (**C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(strVals)) +
				uintptr(pointerSize*C.size_t(outputValueIndex))))

			if firstError == nil && err != nil {
				// Allow for all values to be evaluated to simplify batch cleanup
				*strValPos = C.CString("")
				*length = 0
				firstError = err
			} else {
				*strValPos = C.CString(strVal)
				*length = C.size_t(len(strVal))
			}
		} else {
			*length = 0
			*nullIndicator = C.dbcapi_bool(1)
		}
	}

	return bpv, firstError
}

func (statement *statement) getAdjustedStringValueFromTime(rowIndex int, columnIndex int,
	tVal time.Time) (string, error) {
	var insertTimeVal time.Time
	var strVal string

	if statement.conn.dsnInfo.ConvertTimestampToUTC() {
		insertTimeVal = tVal.UTC()
	} else {
		insertTimeVal = tVal
	}

	paramInfo := statement.retrieveParamInfo(columnIndex)

	if paramInfo == nil {
		return "", fmt.Errorf("Unable to get the Param Info for column %d", columnIndex)
	}

	// Server does not support the year = 0 for DATE and TIMESTAMP type columns
	if (insertTimeVal.Year() == 0) &&
		((paramInfo.native_type == C.dbcapi_native_type(C.DT_DAYDATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_DATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_LONGDATE)) ||
			(paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDDATE))) {
		if statement.conn.dsnInfo.ConvertTimestampToUTC() {
			return strVal, fmt.Errorf(
				"Error for Time value in row %d, column %d: Specified date '%v' would result "+
					"in a UTC date '%v' with year = 0, which is not supported by the server",
				rowIndex, columnIndex, tVal, insertTimeVal)
		}
		return strVal, fmt.Errorf("Error for Time value in row %d, column %d: "+
			"Specified date '%v' with year = 0 is not supported by the server",
			rowIndex, columnIndex, insertTimeVal)
	}

	if paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDTIME) || // TIME
		paramInfo.native_type == C.dbcapi_native_type(C.DT_TIME) {
		strVal = insertTimeVal.Format(TIME_LAYOUT_STRING)
	} else if paramInfo.native_type == C.dbcapi_native_type(C.DT_DAYDATE) || // DATE
		paramInfo.native_type == C.dbcapi_native_type(C.DT_DATE) {
		strVal = insertTimeVal.Format(DATE_LAYOUT_STRING)
	} else if paramInfo.native_type == C.dbcapi_native_type(C.DT_LONGDATE) || // TIMESTAMP
		paramInfo.native_type == C.dbcapi_native_type(C.DT_SECONDDATE) {
		if insertTimeVal.Nanosecond() == 0 {
			strVal = insertTimeVal.Format(SECONDDATE_LAYOUT_STRING)
		} else {
			strVal = insertTimeVal.Format(TS_LAYOUT_STRING)
		}
	} else {
		// Non-DATE|TIME|TIMESTAMP type - let the conversion success/failure be determined by
		// The C layer implementation
		strVal = insertTimeVal.Format(TS_LAYOUT_STRING)
	}
	return strVal, nil
}

func (transaction *transaction) Commit() error {
	commitRes := C.dbcapi_commit(transaction.dbcapi_conn)
	if commitRes != 1 {
		return getError(transaction.dbcapi_conn)
	}

	autoCommRes := C.dbcapi_set_autocommit(transaction.dbcapi_conn, 1)
	if autoCommRes != 1 {
		return getError(transaction.dbcapi_conn)
	}

	err := setIsolationLevel(transaction.dbcapi_conn, DEFAULT_ISOLATION_LEVEL)
	if err != nil {
		return err
	}

	return nil
}

func (transaction *transaction) Rollback() error {
	rollbackRes := C.dbcapi_rollback(transaction.dbcapi_conn)
	if rollbackRes != 1 {
		return getError(transaction.dbcapi_conn)
	}

	autoCommRes := C.dbcapi_set_autocommit(transaction.dbcapi_conn, 1)
	if autoCommRes != 1 {
		return getError(transaction.dbcapi_conn)
	}

	err := setIsolationLevel(transaction.dbcapi_conn, DEFAULT_ISOLATION_LEVEL)
	if err != nil {
		return err
	}

	return nil
}

func (nullTime *NullTime) Scan(value interface{}) error {
	nullTime.Time, nullTime.Valid = value.(time.Time)
	return nil
}

func (nullTime NullTime) Value() (driver.Value, error) {
	if !nullTime.Valid {
		return nil, nil
	}
	return nullTime.Time, nil
}

func (nullBytes *NullBytes) Scan(value interface{}) error {
	nullBytes.Bytes, nullBytes.Valid = value.([]byte)
	return nil
}

func (nullBytes NullBytes) Value() (driver.Value, error) {
	if !nullBytes.Valid {
		return nil, nil
	}
	return nullBytes.Bytes, nil
}

func (bulkInt64 *BulkNullInt64) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)

	if !valid || !theValue.isInteger {
		bulkInt64.Int64s = nil
		bulkInt64.Valids = nil
		return errors.New("Error: Not bulk fetching Int64s")
	}

	bulkInt64.convertRawCArrayToInt64Array(theValue)
	return nil
}

func (bulkInt64 *BulkNullInt64) convertRawCArrayToInt64Array(bulkValues *bulkValue) {
	if bulkInt64.Int64s == nil || len(bulkInt64.Int64s) < int(bulkValues.rowsetFetchSize) {
		bulkInt64.Int64s = make([]int64, bulkValues.rowsetFetchSize)
		bulkInt64.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}
	bulkInt64.NumberOfRows = uint64(bulkValues.numberOfRows)
	int64Size := uint64(bulkValues.precision)
	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		theValuePtr := (*C.int64_t)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) + uintptr(C.size_t(int64Size)*C.size_t(i))))
		var theValue C.int64_t
		C.memcpy(unsafe.Pointer(&theValue), unsafe.Pointer(theValuePtr), C.size_t(int64Size))
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) + uintptr(nullSize*C.size_t(i))))
		bulkInt64.Int64s[i] = int64(theValue)
		bulkInt64.Valids[i] = (*null == 0)
	}
}

func (bulkInt64 *BulkNullInt64) Value() (driver.Value, error) {
	// There is no concept of single validity anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkInt64.Int64s, nil
}

func (bulkFloat64 *BulkNullFloat64) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)

	if !valid || !theValue.isFloat {
		bulkFloat64.Float64s = nil
		bulkFloat64.Valids = nil
		return errors.New("Error: Not bulk fetching Float64s")
	}

	bulkFloat64.convertRawCArrayToFloat64Array(theValue)
	return nil
}

func (bulkFloat64 *BulkNullFloat64) convertRawCArrayToFloat64Array(bulkValues *bulkValue) {
	if bulkFloat64.Float64s == nil || len(bulkFloat64.Float64s) < int(bulkValues.rowsetFetchSize) {
		bulkFloat64.Float64s = make([]float64, bulkValues.rowsetFetchSize)
		bulkFloat64.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}
	bulkFloat64.NumberOfRows = uint64(bulkValues.numberOfRows)
	doubleSize := uint64(bulkValues.precision)
	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		theValuePtr := (*C.double)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) + uintptr(C.size_t(doubleSize)*C.size_t(i))))
		var theValue C.double
		C.memcpy(unsafe.Pointer(&theValue), unsafe.Pointer(theValuePtr), C.size_t(doubleSize))
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) + uintptr(nullSize*C.size_t(i))))
		bulkFloat64.Float64s[i] = float64(theValue)
		bulkFloat64.Valids[i] = (*null == 0)
	}
}

func (bulkFloat64 *BulkNullFloat64) Value() (driver.Value, error) {
	// There is no concept of single validity anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkFloat64.Float64s, nil
}

func (bulkBool *BulkNullBool) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)
	if !valid || ((theValue.dataType != C.A_UVAL8) || (theValue.nativeType != C.DT_BOOLEAN)) {
		bulkBool.Bools = nil
		bulkBool.Valids = nil
		return errors.New("Error: Not bulk fetching Bools")
	}

	bulkBool.convertRawCArrayToBoolArray(theValue)
	return nil
}

func (bulkBool *BulkNullBool) convertRawCArrayToBoolArray(bulkValues *bulkValue) {
	if bulkBool.Bools == nil || len(bulkBool.Bools) < int(bulkValues.rowsetFetchSize) {
		bulkBool.Bools = make([]bool, bulkValues.rowsetFetchSize)
		bulkBool.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}
	bulkBool.NumberOfRows = uint64(bulkValues.numberOfRows)
	boolSize := uint64(bulkValues.precision)
	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		theValuePtr := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) + uintptr(C.size_t(boolSize)*C.size_t(i))))
		var theValue C.dbcapi_bool
		C.memcpy(unsafe.Pointer(&theValue), unsafe.Pointer(theValuePtr), C.size_t(boolSize))
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) + uintptr(nullSize*C.size_t(i))))
		bulkBool.Bools[i] = bool(theValue == 1)
		bulkBool.Valids[i] = (*null == 0)
	}
}

func (bulkBool *BulkNullBool) Value() (driver.Value, error) {
	// There is no concept of single validity anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkBool.Bools, nil
}

func (bulkTime *BulkNullTime) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)

	if !valid || (!theValue.isDate && !theValue.isTime && !theValue.isTimestamp) {
		bulkTime.Times = nil
		bulkTime.Valids = nil
		return errors.New("Error: Not bulk fetching Times")
	}

	var err error

	if theValue.isTimestamp {
		if theValue.isSecondDate {
			err = bulkTime.convertRawCArrayToTimeArray(theValue, SECONDDATE_LAYOUT_STRING)
		} else {
			err = bulkTime.convertRawCArrayToTimeArray(theValue, TS_LAYOUT_STRING)
		}
	} else if theValue.isDate {
		err = bulkTime.convertRawCArrayToTimeArray(theValue, DATE_LAYOUT_STRING)
	} else if theValue.isTime {
		err = bulkTime.convertRawCArrayToTimeArray(theValue, TIME_LAYOUT_STRING)
	}

	return err
}

func (bulkTime *BulkNullTime) Value() (driver.Value, error) {
	// There is no concept of single validity (NULL) anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkTime.Times, nil
}

func (bulkTime *BulkNullTime) convertRawCArrayToTimeArray(bulkValues *bulkValue,
	layoutFormat string) error {
	if bulkTime.Times == nil || len(bulkTime.Times) < int(bulkValues.rowsetFetchSize) {
		bulkTime.Times = make([]time.Time, bulkValues.rowsetFetchSize)
		bulkTime.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}

	bulkTime.NumberOfRows = uint64(bulkValues.numberOfRows)
	stringSize := uint64(bulkValues.precision)
	var err error

	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) +
			uintptr(nullSize*C.size_t(i))))
		valid := (*null == 0)
		bulkTime.Valids[i] = valid

		if valid {
			theValuePtr := (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) +
				uintptr(C.size_t(stringSize)*C.size_t(i))))
			// It is expected that each string should be Null terminated, coming from dbcapi
			// Just in case it isn't, we take the safe route and NULL terminate the provided buffer
			// always, so that C.GoString() does not run off the end of the buffer
			theValuePtrEnd := (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(theValuePtr)) +
				uintptr(C.size_t(stringSize-1))))
			*theValuePtrEnd = 0
			theValue := C.GoString(theValuePtr)
			bulkTime.Times[i], err = time.Parse(layoutFormat, theValue)

			if err != nil {
				break
			}
		}
	}

	return err
}

func (bulkString *BulkNullString) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)

	if !valid || (!theValue.isString && !theValue.isDecimal) {
		bulkString.Strings = nil
		bulkString.Valids = nil
		return errors.New("Error: Not bulk fetching Strings")
	}

	bulkString.convertRawCArrayToStringArray(theValue)
	return nil
}

func (bulkString *BulkNullString) Value() (driver.Value, error) {
	// There is no concept of single validity (NULL) anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkString.Strings, nil
}

func (bulkString *BulkNullString) convertRawCArrayToStringArray(bulkValues *bulkValue) {
	if bulkString.Strings == nil || len(bulkString.Strings) < int(bulkValues.rowsetFetchSize) {
		bulkString.Strings = make([]string, bulkValues.rowsetFetchSize)
		bulkString.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}

	bulkString.NumberOfRows = uint64(bulkValues.numberOfRows)
	stringSize := uint64(bulkValues.precision)

	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) +
			uintptr(nullSize*C.size_t(i))))
		valid := (*null == 0)
		bulkString.Valids[i] = valid

		if valid {
			theValuePtr := (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) +
				uintptr(C.size_t(stringSize)*C.size_t(i))))
			// It is expected that each string should be Null terminated, coming from dbcapi
			// However, there are cases were buffers may not be NULL terminated, for example in
			// these cases:
			// - The server underrepresents the length of the data in the metadata
			// - The buffer is not large enough to hold an entire ending character
			//   (data may be uninitialized there)
			// We can consider doing a continuous loop to get the string data in cases
			// of strings with length > max and then ultimately creating the string from that
			// For now, we take the safe route and NULL terminate the provided buffer always,
			// so that C.GoString() does not run off the end of the buffer
			// Unfortunate side effect: The cases described above could result in a truncated
			// string being created, and there is currently no way to know if a specific value
			// was truncated
			theValuePtrEnd := (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(theValuePtr)) +
		 		uintptr(C.size_t(stringSize-1))))
			*theValuePtrEnd = 0
			theValue := C.GoString(theValuePtr)
			bulkString.Strings[i] = theValue
		}
	}
}

func (bulkBytes *BulkNullBytes) Scan(value interface{}) error {
	theValue, valid := value.(*bulkValue)

	if !valid || !theValue.isBinary {
		bulkBytes.Bytes = nil
		bulkBytes.Valids = nil
		return errors.New("Error: Not bulk fetching Bytes")
	}

	bulkBytes.convertRawCArrayToStringArray(theValue)
	return nil
}

func (bulkBytes *BulkNullBytes) Value() (driver.Value, error) {
	// There is no concept of single validity (NULL) anymore.
	// If Scan() failed, then both of these will be nil, and
	// the application should depend on the Error from Scan() to tell it
	// what happened
	return bulkBytes.Bytes, nil
}

func (bulkBytes *BulkNullBytes) convertRawCArrayToStringArray(bulkValues *bulkValue) {
	if bulkBytes.Bytes == nil || len(bulkBytes.Bytes) < int(bulkValues.rowsetFetchSize) {
		bulkBytes.Bytes = make([][]byte, bulkValues.rowsetFetchSize)
		bulkBytes.Valids = make([]bool, bulkValues.rowsetFetchSize)
	}
	bulkBytes.NumberOfRows = uint64(bulkValues.numberOfRows)
	size := uint64(bulkValues.precision)
	for i := 0; i < int(bulkValues.numberOfRows); i++ {
		null := (*C.dbcapi_bool)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.nulls)) +
			uintptr(nullSize*C.size_t(i))))
		length := (*C.size_t)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.lengths)) +
			uintptr(lengthSize*C.size_t(i))))
		valid := (*null == 0)
		bulkBytes.Valids[i] = valid

		if valid {
			theValuePtr := (*C.char)(unsafe.Pointer(uintptr(unsafe.Pointer(bulkValues.bulkValues)) +
				uintptr(C.size_t(size)*C.size_t(i))))
			theValue := convertToLargeByteArray(theValuePtr, uint(math.Min(float64(*length), float64(size))))
			bulkBytes.Bytes[i] = theValue
		}
	}
}
